/*
Copyright (c) 2007-2020, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <core_config.h>
// Defines

#define FIRST_IPI         10  // GIC interrupts 10-39 are used to signal interrupts between cores.

// Global variables.
volatile int ready[40] = {0};
volatile int start_test[40] = {0};
volatile int boot_count[40] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
							   17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
							   33, 34, 35, 36, 37, 38, 39, 40};

void boston_lcd_display_byte_ab(unsigned int, char);

//
// set_ipi(): Send an inter-processor interrupt to the specified cpu.
//
void set_ipi(int cpu_num) {
    GIC_SH_WEDGE_ADDR = 0x80000000 + FIRST_IPI + cpu_num ; // Use external interrupts 32..39 for ipi
}


//
// main(): Synchronized run of shared test code coordinated by cpu0.
//
int main(unsigned int cpu_num, unsigned int core_num, unsigned int vpe_num, unsigned int num_vpe_this_core, unsigned int num_cores_in_system) {
    
    int i, j, k ;
    char display_char, blank;
    int temp, core, vpe;
    
    // Add the display_cpu_num to save the digit of display
    //	So if we have 4cores*2VPEs then it still OK to use 8 LED on Boston board
    //	Noted all of code have display_cpu_num are all changed 
    int	display_cpu_num;
  	display_cpu_num = (core_num*num_vpe_this_core)+vpe_num;
		
    // End timing of boot for this "cpu".
    asm volatile ("mfc0 %[temp], $9": [temp] "=r"(temp) :) ;
    boot_count[cpu_num] = temp ;

		// cpu0 synchronizes test code execution.
		if (cpu_num == 0) {
			// Wait for other VPEs to indicate they are ready.
		    boston_lcd_display_byte_ab(display_cpu_num, 'W');

			ready[0] = 1;	// don't want to wait for itself;
			for(core = 0; core < num_cores_in_system; core++)
				for(vpe = 0; vpe < num_vpe_this_core; vpe++) {

					while(!ready[(core * 4)+ vpe]);
				}

		    boston_lcd_display_byte_ab(display_cpu_num, 'I');

			// Release other VPEs to run their tasks.
			for(core = 0; core < num_cores_in_system; core++)
				for(vpe = 0; vpe < num_vpe_this_core; vpe++) {
					i = ((core * 3)+ vpe);
					if(i != 0)						// don't send interrupt to itself
						set_ipi(i) ;                // Send the ipi
				}

			// Wait for other VPEs to clear ready to indicate they got the IPI.
			ready[0] = 0;	// don't want to wait for itself;
			for(core = 0; core < num_cores_in_system; core++)
				for(vpe = 0; vpe < num_vpe_this_core; vpe++)
					while(ready[(core * 4)+ vpe]);

		} else {

			GIC_SH_WEDGE_ADDR = FIRST_IPI + display_cpu_num ;    // Clear this "cpu"'s ipi source

			// Enable interrupts and wait to be released via ipi from cpu0
			asm volatile ("ei") ;

			// Other VPE indicate ready and wait...
		    boston_lcd_display_byte_ab(display_cpu_num, 'r');

			asm volatile ("di") ;
			ready[cpu_num] = 1 ;

			while (!start_test[cpu_num]) {
				// This code will only work reliably if the WII bit is set in config7.
				// When this bit is set any interrupt even when they are disabled will cause
				// wait to return. This avoids a race condition
				asm volatile ("wait") ;     // Wait for interrupt (qualified with "start_test").
				// enable interrupts so interrupt routine can run and set the start_test bit
				asm volatile ("ei") ;
				asm volatile ("ehb") ;
				// Disable interrupts again so there is not race condition between testing the
				// start_test bit variable and going back to wait.
				// NOTE for this code we are only expecting IPI that interrupt.
				asm volatile ("di") ;
				asm volatile ("ehb") ;

			}
			ready[cpu_num] = 0 ; // Clear ready so CPU0 will know this CPU has cleared the wait state.

		 }

    // Put test code here:
	asm volatile ("sltiu $r0, $r0, 0xbc2");  // stop code for simulation Passed
	display_char = (char)(display_cpu_num + 0x30);   // ASCII char for cpu number.
    blank = (char)0x20;
		
    while (1) {
        boston_lcd_display_byte_ab(display_cpu_num, display_char);
        for (j = 0 ; j < ((display_cpu_num+1) * 1000000) ; j++)  {
        				    	asm volatile ("nop	": : :) ;
        }
        boston_lcd_display_byte_ab(display_cpu_num, blank); // blank out display position for this "cpu".
        for (j = 0 ; j < ((display_cpu_num+1) * 1000000) ; j++) {
        				    	asm volatile ("nop	": : :) ;
        }
    }
    return 0 ;
}
