/*
 * I7200/start.S
 *
 *  Start of boot code for I7200 Core
*/
/*
Copyright (c) 2007-2020, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <mips/asm.h>
#include <boot.h>
#include <mips/m32c0.h>
#include <mips/regdef.h>
#include <mips/mt.h>
#include <cm2.h>
#ifdef EVA
 	#include <eva.h>
#endif

/**************************************************************************************
    R E S E T   E X C E P T I O N   H A N D L E R
**************************************************************************************/


LEAF(__reset_vector)
    la a2,     boot_code
    mtc0	zero, C0_COUNT  	    // Clear cp0 Count (Used to measure boot time.)
    jr a2

	// Note: adding code here may conflict with board ID register at 0xbfc0010.

END(__reset_vector)


/**************************************************************************************
    B O O T   E X C E P T I O N   H A N D L E R S
**************************************************************************************/
.org 0x180
#ifdef MPU
	 j   General_Exception

#else
    	sdbbp	// if probe attached break to debugger
#endif
.org 0x200 /* TLB refill, 32 bit task. */
#ifdef MPU
	//  The MPU doesn't have kesg0, kesg1 so the exception must redirect to MPU RAM to handle
	//  And remove Status.BEV to let Ebase work
	//  Because Status.BEV=0, so the interrupt vector points to offset 0x200
	//  We need to handle the interrupt here:
	 j   General_Exception

#else
    	sdbbp	// if probe attached break to debugger
#endif

.org 0x280 		// XTLB refill, 64 bit task
    sdbbp		// if probe attached break to debugger
.org 0x300		// Cache error exception
    sdbbp		// if probe attached break to debugger

.org 0x380 /* General exception. */
General_Exception:
	// expecting an interprocessor interrupt
	mfc0	k1, C0_CAUSE
	ext		k0, k1, 2, 5				// extract exit code
	bne		k0, zero, other_code		// should be 0 indicating a interrupt

	ext		k0, k1, 10, 5				// extract interrupt
	li		k1, 1
	bne		k0, k1, other_code			// should be interrupt 0 if not go to other_code


	// process the interprocessor interrupt

	// First, need to calculate the number of the interrupt that is connected
	// to this VPE.  The interrupt number is calculated as follows:
	// number = CORENUM * 3 + VPENUM + base IPI number
    lw      k0, (CORE_LOCAL_CONTROL_BLOCK + GCR_CL_ID) (r22_gcr_addr) // Get core number
    li      k1, 3
    mul     k1, k0, k1					// CORENUM * 3
    mfc0    k0, C0_TCBIND				// Get TCBind.  Contains CurVPE
    ext     k0, k0, 0, 2				// Extract lower two bits of CurVPE
    addu    k1, k0, k1					// Now k1 has CORENUM * 3 + VPE
    addiu   k1, 0xa						// Offset to base of IPI interrupts.

    li      k0, (GIC_SH_WEDGE | GIC_BASE_ADDR_PB) // get address of write edge register
    sw      k1, 0(k0)					// Clear this IPI by writing to the write edge register

    // Write a 1 to the element in the start test array for this processor
	// once the code returns to normal processing (in main()) it will check the start[CPUNum]
	// to see if it should break out of the idle loop and continue with its test code
    la      k0, start_test				// load the address of the start_test array
    mfc0    k1, C0_EBASE				// Get cp0 EBase
    ext     k1, k1, 0, 10				// Extract CPUNum
    sll     k1, k1, 2					// x 4 for integer element
    addu    k0, k0, k1					// index into start_test array
    li      k1, 1
    sw      k1, 0(k0)					// Release "cpu"/vpe to execute "C" test code.

   	eret

other_code:
	// unexpected exception read the C0_CAUSE and C0_EPC so it will appear in a trace file
	// Just to make it easier to see what's in the registers
    mfc0    k1, C0_CAUSE
    mfc0	k0, C0_EPC

    sltiu	zero, zero, 0xbc1	// if using simulator stop with simulator fail condition
	sdbbp 						// if probe attached break to debugger

.org 0x480 /* EJTAG debug exception (EJTAG Control Register[ProbTrap] == 0.) */
	// Write to Boston display indicating a exception
 	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

    sltiu	zero, zero, 0xbc1	// if using simulator stop with  simulator fail condition
1:  b       1b  				// if no probe attached just spin


/**************************************************************************************
**************************************************************************************/
.org 0x500 /* Resume code past the boot exception vectors. */

boot_code:
#ifdef MPU
	// For a I7200 that is using the MPU instead of a TLB the MPU will need to
	// be initialized.  Note: The processor boots in FMT mode this boot code is
	// loaded into a kuseg address which is mapped VA=PA for kuseg only.
	// After enable MPU then MPU ALL of memory is mapped VA=PA

    // Only VPE0 can initialize the MPU
    mfc0    a0, C0_EBASE
    andi    a0, 3					// clear all but bit 0~1 EBase[CPUNum][bit 0~1] = VPE number
    bnez    a0, MPU_EnableMPU_Done	// 0 for VPE0 if not branch to MPU_EnableMPU_Done


MPU_EnableMPU:
    li      a0, (CDMM_P_BASE_ADDR>>4)|(1<<10)   // load CDMM register base address
    mtc0    a0, C0_CDMMBASE
    ehb                                         // ehb remove cp0 hazard
    li      a0, CDMM_P_BASE_ADDR                // load a0 with address for CDMM
    lw      a1, MPU_CDMM_OFFSET+MPU_Config(a0)
    li      a2, (1<<31)                         // MPU_Config bit 31 = enable
    or      a1, a1, a2
    sw      a1, MPU_CDMM_OFFSET+MPU_Config(a0)  // Enable MPU
    sync

MPU_EnableMPU_Done:
// MPU is enabled so all memory is mapped 1to1,  VA/PA
#endif

    mfc0    a0, C0_STATUS				// Read CP0 Status
    bbeqzc  a0, 19,	verify_isa			// Branch if this is NOT an NMI exception.

	// NMI was set so stop, exit to debugger or process debug interrupt
    // (otherwise processor will go into an endless boot loop)
    sdbbp

verify_isa: // Verify device ISA meets code requirements (MIPS32R2 or later.)
	#if	defined(BOSTON_RAM) || defined(BOSTON_RAM_MPU)  // make sure ram is up before checking array
		LA 	a2, init_FPGA_mem		// Initialize the ROC-it2 MC (Memory Controller.)
		jalr 	a2

	#endif
    mfc0    a0, C0_CONFIG				// Read CP0 Config
    srl     a0, 10						// Shift [AT AR] into LSBs.
    andi    a3, a0, 0x18				// Inspect CP0 Config[AT]
    beqz    a3, is_mips32				// Branch if executing on MIPS32 ISA.

    sdbbp 								// Failed assertion: MIPS32R2.

is_mips32:
    andi    a3, a0, 0x07				// Inspect CP0 Config[AR]
    bnez    a3, init_common_resources	// Continue if ISA is MIPS32R2 or later.

    sdbbp								// Failed assertion MIPS32R2.
/**************************************************************************************
 What is initialized on execution depends on the core/vpe executing it.
 (A non-MT device is treated as tc0/vpe0, non-CMP device is treated as core0.)
**************************************************************************************/
init_common_resources:  // initializes resources for virtual or physical "cpu".
    la a2,init_gpr		// Fill register file with set  value.
	jalr a2

    la a2,	set_gpr_boot_values  // Fill register file boot info.
	jalr a2

    la a2,     init_cp0    // Init CP0 Status, Count, Compare, Watch*, and Cause.
    jalr a2
	#ifndef MPU
	 	// Determine if we have a TLB
		mfc0    a2, C0_CONFIG			// read C0_Config
		ext	    a2, a2, 7, 3			// extract MT field
		li	    a3, 0x1					// load a 1 to check against
		bne	    a2, a3, done_tlb		// no tlb?

	    la a2,  init_tlb    			// Generate unique EntryHi contents per entry pair.
	    jalr a2
done_tlb:
	#endif

    la a2,	init_gic    // Configure the global interrupt controller.
    jalr a2

    bnez    r9_vpe_num, init_done // If we are not a vpe0 then we are done.

init_core_resources:    // VPE0 on all cores will execute init_core_resources.


	// Initialize the L2 cache for the cluster
	move	a3, zero
	bnez    r8_core_num, init_L1_icache	//  Only done from core 0.

	li		a3, 1							// set for L2 initialization in progress
	li		a2, L2SM_COP_CMD_START			// NOTE command type is 0x0: Index WB Inv/Index Inv (Full cache flush) and Start Command "1" is first 2 bits
	sw      a2, GCR_L2SM_COP(r22_gcr_addr)	// Write L2 Configuration register

 	// Initialize the L1 instruction cache
init_L1_icache:
	li 		a1, 0x8 			//invalidate whole icache
	mtc0 	a1, $22, 0 			//C0_BCOP
	ehb

	// Initialize the L1 data cache
	li 		a0, 0x9				//invalidate whole dcache
	mtc0 	a0, $22, 0	 		//C0_BCOP
	ehb

// Wait for L2 initialization to finish
	beqz	a3, Cache_On		// L2 initialization in progress?
L2_Running:
	lw		a3, GCR_L2SM_COP(r22_gcr_addr)
	ext		a0, a3, L2SM_COP_MODE_SHIFT, L2SM_COP_MODE_BITS
	bnez	a0, L2_Running

Cache_On:

	#if defined (EVA) || defined (MPU)
		// Only for cores built to boot in EVA or MPU mode
		// continue in uncached mode
	#else
		// The changing of Kernel mode cacheability must be done from KSEG1
		// Since the code is executing from KSEG0 It needs to do a jump to KSEG1 change
		// K0 and jump back to KSEG0
		la 		a2,		change_k0_cca
		li 		a1, 0xf
		ins		a2, a1, 29, 1 // changed to KSEG1 address by setting bit 29
		jalr	a2

	#endif

    la a2,     init_itc    // Initialize Inter-Thread Communications unit
    jalr a2

    bnez    r8_core_num, init_sys_resources_done // Only core0/vpe0 needs to init systems resources.

init_sys_resources:     // Only done by core0 vpe0.
    la a2,     init_cpc    // Initialize the CPS CPC (Cluster Power Controller.)
    jalr a2

	// Allow each core access to the CM registers (they should only access their local registers.)
	li	    a0, 2							// Start building mask for cores in this cps.
    sll     a0, a0, r19_more_cores
    addiu   a0, -1							// Complete mask.
	sw	    a0, GCR_ACCESS(r22_gcr_addr)	// GCR_ACCESS

    la a2,     copy_c2_ram // Copy "C" code and data to RAM and zero bss (uncached.)
    jalr a2

    beqz    r11_is_cps, init_vpe_next // UP Core so skip multi core stuff

release_cores:

    la a2,     release_mp  // Release other cores to execute this boot code.
    jalr a2

init_sys_resources_done:		// All Cores (VPE0)

	#ifdef MPU
		//  Since the icache has been initialized change the MPU CCA setting to cacheable
	    li      a0, CDMM_P_BASE_ADDR                        // a0 have kseg1 address for CDMM
	    li      a1, (0x2)<<24|(0x2)<<16|(0x2)<<8|(0x5)      // Segment 0 set to CWB (Cacheable, coherent, write-back, write-allocate, read misses request Shared)
	    sw      a1, MPU_CDMM_OFFSET+MPU_SegmentControl0(a0) // Segment 0 address 0~0x0FFFFFFF to cacheable
	    sync
	#endif

    la a2,     join_domain // Join the Coherence  domain. (OK to use D$ on return.)
    jalr a2

init_vpe_next:
    la a2,     init_vpe_s   // Set up MT ASE vpe1 to execute this boot code also.
    jalr a2

init_done:
	#ifdef EVA

		// For non-EXL Kernel mode - Segments CFG5 and CFG4 are directly mapped to the lower 2GB of the physical address space encompassing the 2 RAM memory blocks and the I/O space to
		// be accessed as coherent cached exclusive on write.
		// For user and supervisor modes - Segments CFG5 and CFG4 are mapped through the TLB
		li		a0, (SEGCTL_CFG4_PA_4|SEGCTL_CFG5_PA_0|SEGCTL_CFG4_AM_MUSUK|SEGCTL_CFG5_AM_MUSUK|SEGCTL_CFG4_EU_MASK|SEGCTL_CFG5_EU_MASK|SEGCTL_CFG4_C_CWBE|SEGCTL_CFG5_C_CWBE)
		mtc0 	a0, C0_SEGCTL2
		// Set CFG1 and CFG0 to be UnMapped Kernel to Physical address 0x00000000 and 0x20000000 respectively both uncached. This will be used to address the CM registers and
		// can be used for I/O devices. NOTE: This is needed to access the CM registers uncached
		li		a0, (SEGCTL_CFG2_PA_0|SEGCTL_CFG3_PA_0|SEGCTL_CFG2_AM_UK|SEGCTL_CFG3_AM_UK|SEGCTL_CFG2_EU_MASK|SEGCTL_CFG3_EU_MASK|SEGCTL_CFG2_C_UC|SEGCTL_CFG3_C_WB)
	    mtc0 	a0, C0_SEGCTL1
		ehb

	#endif
	#ifdef MPU
	//  Becasue the MPU doesn't have a kesg0 or kesg1 the exception vector must redirect to MPU RAM
	//  Here set EBASE to the MPU RAM address and remove Status.BEV to let Ebase work
	MPU_Set_Ebase:
	    la      a0, __reset_vector      // EBase to reset vector
	    ori     a0, (1<<11)             // MPU Ebase need set bit 31~30 enable WG bit
	    mtc0    a0, C0_EBASE
	    ehb

	    //Remove Status.BEV
	    mfc0    a0, C0_STATUS
	    li      a1, ~(1<<22)
	    and     a0, a0, a1
	    mtc0    a0, C0_STATUS
	    ehb
	#endif

    // Prepare for eret to main (sp and gp set up per vpe in init_gpr).
    la      ra, all_done	// If main return then go to all_done:.
    la      a1, main
    mtc0    a1, C0_ERRPC 			// ErrorEPC
    // Prepare arguments for main()
    move    a0, r23_cpu_num			// main(arg0) is the "cpu" number (cp0 EBase[CPUNUM].)
    move    a1, r8_core_num			// main(arg1) is the core number.
    move    a2, r9_vpe_num			// main(arg2) is the vpe number.
    addiu   a3, r20_more_vpes, 1	// main(arg3) is the number of vpe on this core.
    addiu 	a4, r19_more_cores, 1	// main(arg4) total cores in system NOTE destroys r8_core_num
    eret    // Exit reset exception handler for this vpe and start execution of main().

/**************************************************************************************
**************************************************************************************/
all_done:
	sltiu zero, zero, 0xbc1	# simulator fail condition
	// Looks like main returned. Just busy wait spin.
    b       all_done

#if defined (EVA) || defined (MPU)
#else
LEAF(change_k0_cca)
	// NOTE! This code must be executed in KSEG1 (not KSGE0 uncached)
    // Set CCA for kseg0 to cacheable
	mfc0	a0, C0_CONFIG	// read C0_Config
	li	    a1, 3			// CCA for all others
	beqz    r11_is_cps, set_kseg0_cca
 	li	    a1, 5			// CCA for coherent cores (fall through)

set_kseg0_cca:
    ins	    a0, a1, 0, 3	// insert K0
	mtc0	a0, C0_CONFIG	// write C0_Config
	jalr.hb   zero, ra

END(change_k0_cca)
#endif
