/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <mips/regdef.h>
#include <boot.h>
#include <mips/m32c0.h>
#include <sead.h>
#include <lcd.h>

.set nomicromips							# This causes the code below to be compiled as MIPS32

LEAF(__reset_vector)
	la a2,	check_nmi
	mtc0	$0, C0_COUNT  	    # Clear cp0 Count (Used to measure boot time.)
	jr a2

    # Note: adding code here will conflict with board ID register at 0xbfc0010.

END(__reset_vector)

/**************************************************************************************
    B O O T   E X C E P T I O N   H A N D L E R S
**************************************************************************************/
/* NOTE: the linker script must insure that this code starts at start + 0x200 so the exception */
/* vectors will be addressed properly. All .org assume this! */
/* TLB refill, 32 bit task. */

.org 0x200 /* TLB refill, 32 bit task. */
tlb_refill_excpt:
    sdbbp

.org 0x280 /* XTLB refill, 64 bit task. start + 0x280*/
xtlb_refill_excpt:
    sdbbp

.org 0x300 /* Cache error exception. start + 0x300 */
cache_error_excpt:
    sdbbp

.org 0x380 /* General exception. start + 0x380*/
general_excpt:
	DISP_STR( msg_gexcpt );
    eret

.org 0x480 /* EJTAG debug exception (EJTAG Control Register[ProbTrap] == 0.) start + 0x480*/
debug_excpt:
	b		debug_excpt  /* Stay here */

/**************************************************************************************
**************************************************************************************/
.org 0x500 /* Resume code past the boot exception vectors. */

.set micromips							# This causes the code below to be compiled as micromips

LEAF(check_nmi)
check_nmi: # Verify we are here due to a reset (and not NMI.)

/* If you want a place to attach to a running target uncomment this loop
once you attach the debugger halt the cpu and set the PC to the address of the nop
in the loop. From that point you can go or step or anything else in the debugger
*/
/*
2:	b 2b
*/

    mfc0    s1, C0_STATUS              	# Read CP0 Status
    ext	    s1, s1, 19, 1				# extract NMI
    beqz    s1, verify_isa            	# Branch if this is NOT an NMI exception.
    sdbbp                              	# Failed assertion: NMI.
END(check_nmi)

verify_isa: # Verify device ISA meets code requirements (MIPS32R2 or later.)
    mfc0    s1, C0_CONFIG              	# Read CP0 Config
    ext	    s0, s1, 13, 2				# extract AT
    beqz    s0, is_mips32             	# Branch if executing on MIPS32R2 ISA.
    sdbbp                               # Failed assertion: MIPS32R2.

is_mips32:
    ext	    s0, s1, 10, 3				# extract AR
    bnez    s0, init_common_resources 	# Continue if ISA is MIPS32r2 or later.
    sdbbp                               # Failed assertion MIPS32R2.
/**************************************************************************************
 What is initialized on execution depends on the core/vpe executing it.
 (A non-MT device is treated as tc0/vpe0, non-CMP device is treated as core0.)
**************************************************************************************/
init_common_resources:  # initializes resources for "cpu".
    la a2,	init_gpr
	jalr a2

    la a2,	set_gpr_boot_values  # Fill register file boot info.
	jalr a2

    # Clear LCD Display
	li    s0, KSEG1(SEAD3_LCD)
	SEAD3_LCD_CMD(s0,s1,LCD_CLEAR)

	// Display boot message
	DISP_STR( msg_boot );

    la a2,     init_cp0    # Init CP0 Status, Count, Compare, Watch*, and Cause.
    jalr a2

    la a2,     init_tlb    # Generate unique EntryHi contents per entry pair.
    jalr a2

    la a2,     init_icache # Initialize the L1 instruction cache. (Executing using I$ on return.)
    jalr a2

// The changing of Kernel mode cacheability must be done from KSEG1
// Since the code is executing from KSEG0 It needs to do a jump to KSEG1 change K0 and jump back to KSEG0
	li t3, 0	# clear t3/r11 to tell change_k0_cca that this is a non-cps core
	la a2,		change_k0_cca
	li      	a1, 0xf
	ins     	a2, a1, 29, 1 // changed to KSEG1 address by setting bit 29
    jalr a2

    la a2,     init_dcache # Initialize the L1 data cache
    jalr a2

    la a2,     copy_c2_ram # Copy "C" code and data to RAM and zero bss (uncached.)
    jalr a2

    # Prepare for eret to main (sp and gp set in set_gpr_boot_values).
    la      ra, all_done	# If main returns then go to all_done:.
    la      v0, main		# load the address of the main function
    mtc0    v0, $30			# Write ErrorEPC with the address of main
    ehb						# clear hazards (makes sure write to ErrorPC has completed)

    # Return from exception will now execute code in main
    eret    # Exit reset exception handler and start execution of main().


/**************************************************************************************
**************************************************************************************/
all_done:
    # Looks like main returned. Just busy wait spin.
    b       all_done

// messages
// example DISP_STR( msg_cpu )

.text

MSG( msg_gexcpt,	    "G_EXCEPT" )
MSG( msg_boot,	    "BOOT" )
