/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// Defines
#define GIC_SH_WEDGE      *((volatile unsigned int*) (0xbbdc0280))
#define FIRST_IPI         32  // GIC interrupts 32+ are used to signal interrupts between cores.

// Global variables.
volatile int num_cores ;
volatile int ready[8] ;
volatile int start_test[8] ;
volatile int boot_count[8] = { 1, 2, 3, 4, 5, 6, 7, 8};

void boston_lcd_display_byte_ab(unsigned int, char);

//
// set_ipi(): Send an inter-processor interrupt to the specified cpu.
//
void set_ipi(int cpu_num) {
    GIC_SH_WEDGE = 0x80000000 + FIRST_IPI + cpu_num ; // Use external interrupts 32..39 for ipi
}

//
// main(): Synchronized run of shared test code coordinated by cpu0.
//
int main(unsigned int cpu_num) {
    
    int i, j, k ;
    int num_cpus = 0 ;
    char display_char, blank;
    int temp;

    // End timing of boot for this "cpu".
    asm volatile ("mfc0 %[temp], $9": [temp] "=r"(temp) :) ;
    boot_count[cpu_num] = temp ;

    // cpu0 synchronizes test code execution.
    if (cpu_num == 0) {
    	num_cpus = num_cores;
        // Wait for other cpu to indicate they are ready.
        for (i = 1; i < num_cpus; i++) {
        	boston_lcd_display_byte_ab(cpu_num, 'W');	// 'W' for Waiting..
            while (!ready[i]) ;         // Busy wait for all cpu to be ready.
        }
        
        // Release other cpu to run their tasks.
        for (i = 1; i < num_cpus; i++) {
        	boston_lcd_display_byte_ab(i, 'i');		// display 'i' for interrupted (from cpu0.)
            set_ipi(i);								// Send the ipi
        }

    } else {

        GIC_SH_WEDGE = FIRST_IPI + cpu_num ;    // Clear this "cpu"'s ipi source

        // Enable interrupts and wait to be released via ipi from cpu0
        asm volatile ("ei") ;					// smm check: verify ipi source is cleared?

        // Other cpu indicate ready and wait...
        boston_lcd_display_byte_ab(cpu_num, 'r');	// display 'r' for ready.
        ready[cpu_num] = 1 ;
        asm volatile ("di") ;

        while (!start_test[cpu_num]) {
        	// This code will only work reliably if the WII bit is set in config7.
        	// When this bit is set any interrupt even when they are disabled will cause
        	// wait to return. This avoids a race condition
            asm volatile ("wait") ;     // Wait for interrupt (qualified with "start_test").
            // enable interrupts so interrupt routine can run and set the start_test bit
            asm volatile ("ei") ;
            asm volatile ("ehb") ;
            // Disable interrupts again so there is not race condition between testing the
            // start_test bit variable and going back to wait.
            // NOTE for this code we are only expecting IPI that interrupt.
            asm volatile ("di") ;
            asm volatile ("ehb") ;

        }
     }

    // Put test code here:
    display_char = (char)(cpu_num + 0x30);   // ASCII char for cpu number.
    blank = (char)0x20;

    while (1) {
        for (j = 0 ; j < (4 * 8192) ; j++) {
        	boston_lcd_display_byte_ab(cpu_num, display_char);
        }
        for (j = 0 ; j < (4 * 8192) ; j++) {
        	boston_lcd_display_byte_ab(cpu_num, blank); // blank out display position for this "cpu".
        }
    }
    
    return 0 ;
}
