/*
 * init_L2_CM2.S
 *
 */
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/m32c0.h>
#include <core_config.h>

#define LINE_SIZE			$3
#define BYTES_PER_LOOP 		$2
#define SET_SIZE			$4
#define ASSOC				$5
#define CONFIG_L2 			$6
#define END_ADDR			$7
#define TOTAL_BYTES			$12
#define	CURRENT_ADDR		$13
#define GCR_BASE			$16
#define TEMP				$17

#define LINES_PER_ITER 8 // number of cach instructions per loop

/**************************************************************************************
**************************************************************************************/
LEAF(disable_L2)

	bnez    r8_core_num, done_disable_L2	# Only done from core 0.
    // Use CCA Override disable the L2 cache
    // NOTE: If you have a L3 cache you must add code here
    // to disable it or initialize it if it can't be disabled.
    // Disable the L2 cache using CCA override by writing a 0x50 to
    // the GCR Base register. 0x50 enables the CCA override bit and sets
    // the CCA to uncached.

	lw	    GCR_BASE, 0x0008(r22_gcr_addr)  // Read GCR_BASE
    li      TEMP, 0x50                  	// Enable CCA and set to uncached
    ins     GCR_BASE, TEMP, 0, 8    		// Insert bits
    sw      GCR_BASE, 0x0008(r22_gcr_addr)  // Write GCR_BASE

done_disable_L2:
    jr      ra
END(disable_L2)
/**************************************************************************************
**************************************************************************************/
LEAF(enable_L2)

	bnez    r8_core_num, done_enable_L2	# Only done from core 0.
    // Use CCA Override disable the L2 cache
    // NOTE: If you have a L3 cache you must add code here
    // to enable it or initialize it if it can't be enabled.


	lw	    GCR_BASE, 0x0008(r22_gcr_addr)  // Read GCR_BASE
 	ins     GCR_BASE, zero, 0, 8    		  // CCA Override disabled
    sw      GCR_BASE, 0x0008(r22_gcr_addr)  // Write GCR_BASE

done_enable_L2:
    jr      ra
END(enable_L2)
/**************************************************************************************
* Initialize the L2
**************************************************************************************/
LEAF(init_L2)

	// Can be skipped if Config7[HCI] set
	mfc0    TEMP, C0_CONFIG, 7						// Read CP0 Config7
	ext		TEMP, TEMP, HCI, 1					// extract HCI
	bne		TEMP, zero, done_L2_cach_init

	bnez    r8_core_num, done_L2_cach_init			# Only done from core 0.
	// L2 Cache initialization routine
	// Check L2 cache size
	mfc0	CONFIG_L2, C0_CONFIG2		// Read C0_Config2
	// Isolate L2$ Line Size
	ext	    LINE_SIZE, CONFIG_L2, 4, 4		// extract L2 line size

	// Skip ahead if No L2$
	beq	    LINE_SIZE, zero, done_L2_cach_init

	li	    TEMP, 2
	sllv	LINE_SIZE, TEMP, LINE_SIZE			// Now have true L2$ line size in bytes

	// Isolate L2 Sets per Way (cache lines per way)
	ext	    SET_SIZE, CONFIG_L2, 8, 4		// extrace sets per way encoding
	li	    TEMP, 64
	sllv	SET_SIZE, TEMP, SET_SIZE			// L2$ Sets per way

	// Isolate L2 Associativity (number of ways)
	// L2$ Assoc (-1)
	ext	    ASSOC, CONFIG_L2, 0, 4		// extract ways encoding
	addiu	ASSOC, ASSOC, 1			// Decode L2 number of ways

	li 	TEMP, (LINES_PER_ITER)

#ifdef R6
	dmul	SET_SIZE, SET_SIZE, ASSOC				// Total number of sets
	dmul	TOTAL_BYTES, SET_SIZE, LINE_SIZE		// Total number of bytes
	dmul	BYTES_PER_LOOP, LINE_SIZE, TEMP		// Total bytes per loop
#else
	mul	    SET_SIZE, SET_SIZE, ASSOC				// Total number of sets
	mul		TOTAL_BYTES, SET_SIZE, LINE_SIZE		// Total number of bytes
	mul		BYTES_PER_LOOP, LINE_SIZE, TEMP		// Total bytes per loop
#endif

	// Set the starting address at the beginning of kgeg0 (0x80000000) which will corresponds to
	// way 0 index 0 of the cache and position so starting address is in the
	// middle of the first bytes per loop because the code will use + and - offsets
	lui	    CURRENT_ADDR, 0x8000
	srl		TEMP, BYTES_PER_LOOP, 1
	addu	CURRENT_ADDR, TEMP, CURRENT_ADDR

	addu	END_ADDR, CURRENT_ADDR, TOTAL_BYTES		// make ending address
	subu	END_ADDR, END_ADDR, BYTES_PER_LOOP		// -1 bytes per loop interration

	// Clear L23TagLo/L23TagHi registers these are used to set the cache tag
	mtc0	zero, C0_TAGLO, 4
	mtc0	zero, C0_TAGHI, 4	// Note: not all implementations will have a L23 tag Hi but writes will be ignored if not present
	ehb

// due to offset field restrictions code assumes line size will not be more that 128 bytes for a r5 or less core
// or not more than 64 bytes for a r6 and greater core

next_L2_cache_tag:
	// Index Store Tag Cache Op
	// Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
    cache 0xB, (L2LINE_SIZE*-2)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-1)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*0)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*1)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-4)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-3)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*2)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*3)(CURRENT_ADDR)

	addu	    CURRENT_ADDR, BYTES_PER_LOOP			// Get next starting line address
	bge	    	END_ADDR, CURRENT_ADDR, next_L2_cache_tag 	// Done yet?

done_L2_cach_init:
    jr      ra
END(init_L2)

#undef LINE_SIZE
#undef BYTES_PER_LOOP
#undef SET_SIZE
#undef ASSOC
#undef CONFIG_L2
#undef END_ADDR
#undef TOTAL_BYTES
#undef	CURRENT_ADDR
#undef TEMP
#undef GCR_BASE
#undef LINES_PER_ITER
