/*
 * init_L2_CM3.S
 *
 */
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/m32c0.h>
#include <core_config.h>
#include <cps_CM3.h>

	.set	noat			// Don't allow the assembler to use r1(at) for synthetic instr.

#define LINE_SIZE_v1		v1
#define BYTES_PER_LOOP_v0 	v0
#define SET_SIZE_a0			a0
#define ASSOC_a1			a1
#define	CONFIG_L2_a2		a2
#define END_ADDR_a3			a3
#define TOTAL_BYTES			$12	// t4 o32 a4 n32/64
#define	CURRENT_ADDR		$13	// t5 o32 a5 n32/64
#define TEMP_s1				s1

#define LINES_PER_ITER 8	// number of cache instructions per loop

/**************************************************************************************
* Initialize the L2
**************************************************************************************/
LEAF(init_L2)

	bnez	r8_core_num, done_L2_cache_init				// Only done from core 0.

	// The HCI_DONE bit in GCR_L2_RAM_CONFIG may not properly indicate that the
	// automatic hardware cache initialization for L2 is complete.  Do software L2
	// cache initialization instead with CACHE ops.

	// Disable the L2 cache by setting the L2_Bypass bit in the CM3 L2 Configuration
	// register.  This was already done in start.S, but do it here in case this
	// init_L2 routine is ever used somewhere else.  It's OK to set L2 Bypass again
	// even if it was already set.
	ld		CONFIG_L2_a2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
	dli		TEMP_s1, 0x1								// set to uncached (bypass)
	dins	CONFIG_L2_a2, TEMP_s1, 20, 1    			// Insert bits
	sd		CONFIG_L2_a2, GCR_L2_CONFIG(r22_gcr_addr)	// Write L2 Configuration register
	ld		CONFIG_L2_a2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
	sync

	// Isolate L2$ Line Size
	dext	LINE_SIZE_v1, CONFIG_L2_a2, 8, 4			// extract LINE_SIZE

	// Skip ahead if No L2$
	beq		LINE_SIZE_v1, zero, done_l2

	dli		TEMP_s1, 2
	dsllv	LINE_SIZE_v1, TEMP_s1, LINE_SIZE_v1			// decode for true L2$ line size in bytes

	// Isolate L2$ Sets per Way
	dext	SET_SIZE_a0, CONFIG_L2_a2, 12, 4			// extract SET_SIZE_a0
	dli	    TEMP_s1, 64
	dsllv	SET_SIZE_a0, TEMP_s1, SET_SIZE_a0			// decode for sets per way

	// Isolate L2$ Associativity
	dext	ASSOC_a1, CONFIG_L2_a2, 0, 4				// extract ASSOC_a1
	daddiu	ASSOC_a1,ASSOC_a1, 1						// decode for # of ways

	dli 	TEMP_s1, (LINES_PER_ITER)

	dmul	SET_SIZE_a0, SET_SIZE_a0, ASSOC_a1			// Get total number of sets in L2
	dmul	TOTAL_BYTES, SET_SIZE_a0, LINE_SIZE_v1		// Total number of bytes
	dmul	BYTES_PER_LOOP_v0, LINE_SIZE_v1, TEMP_s1	// Total bytes per loop

	// Set the starting address at the beginning of kgeg0 (0x80000000) which will corresponds to
	// way 0 index 0 of the cache and position so starting address is in the
	// middle of the first bytes per loop because the code will use + and - offsets
	dli		CURRENT_ADDR, 0x80000000
	dsrl	TEMP_s1, BYTES_PER_LOOP_v0, 1
	daddu	CURRENT_ADDR, TEMP_s1, CURRENT_ADDR

	daddu	END_ADDR_a3, CURRENT_ADDR, TOTAL_BYTES		// make ending address
	dsubu	END_ADDR_a3, END_ADDR_a3, BYTES_PER_LOOP_v0	// -1 bytes per loop interration

	// Clear L2 Tag registers
	sd 		zero, 0x600(r22_gcr_addr)					// GCR_L2_TAG_ADDR
	sd 		zero, 0x608(r22_gcr_addr)					// GCR_L2_TAG_STATE
	sd 		zero, 0x610(r22_gcr_addr)					// GCR_L2_DATA

next_L2_cache_tag:
	// L2$ Index Store Tag Cache Op
	// Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
	cache 0xB, (L2LINE_SIZE*-2)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*-1)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*0)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*1)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*-4)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*-3)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*2)(CURRENT_ADDR)
	cache 0xB, (L2LINE_SIZE*3)(CURRENT_ADDR)
	daddu	CURRENT_ADDR, BYTES_PER_LOOP_v0					// Get next starting line address
	bgeuc	END_ADDR_a3, CURRENT_ADDR, next_L2_cache_tag	// Done yet?

done_L2_cache_init:
	// Clear L2 Bypass (enable L2)
	ld		CONFIG_L2_a2, GCR_L2_CONFIG(r22_gcr_addr)				// Read L2 Configuration register
	dins	CONFIG_L2_a2, zero, 20, 1    							// Insert bit
	sd		CONFIG_L2_a2, GCR_L2_CONFIG(r22_gcr_addr)				// Write L2 Configuration register

done_l2:
	jalr	zero, ra
END(init_L2)
