/*
 * cm3.h
 *
*/
/*
Copyright (c) 2007-2019, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


/*
 * This include file contains #defines for the memory mapped registers in a coherent Processing system
 * of both single cores and multi threaded cores. It contains registers offset for all the registers,
 * defines for the fields with in the registers and encodings for some of the fields.
 *
 * The fields in the registers are defined by a pair of #defines, one define is the starting bit position of
 * the field ( with a "_SHIFT" appended to the name) and another, (with a "_BITS" appended to the name), is the size of the field. Here is an example of
 * how you would use these #defines in the the extraction of a filed:
 *
 *   li      $5, GIC_BASE_ADDR			         # load GIC KSEG0 Address
 *   lw      $4, GIC_SH_CONFIG($5)               # Read the GIC_SH_CONFIG Register
 *   ext     $4, NUMINTERRUPTS_SHIFT, NUMINTERRUPTS_BITS  # Extract NUMINTERRUPTS
 *
 *   The names for the registers and fields are usually the same as used in the Software Users Manual
 *   (SUM). The exceptions occur when the names would conflict with each other in that case the name
 *   is appended with something to make it unique.
 *
 *
 *   Note this file covers both CM 3 and CM3.5. Differences are noted in the comments.
 */

#ifndef _CM3_H_
#define _CM3_H_

// GCR Offset for GCR_CONFIG, field positions and field size
#define GCR_CONFIG 0x0000
#define ITU_PRESENT_SHIFT 31
#define ITU_PRESENT_BITS 1
#define NUM_CLUSTERS_SHIFT 23
#define NUM_CLUSTERS_BITS 7
#define NUMAUX_SHIFT 20
#define NUMAUX_BITS 3
#define ADDR_REGIONS_SHIFT 16
#define ADDR_REGIONS_BITS 4
#define NUMIOCU_SHIFT 8
#define NUMIOCU_BITS 4
#define PCORES_SHIFT 0
#define PCORES_BITS 8

// GCR Offset for GCR_BASE, field positions and field size
#define GCR_BASE 0x0008
#define GCR_BASE_ADDR_SHIFT 15
#define GCR_BASE_ADDR_BITS 33

// GCR Offset for GCR_CONTROL, field positions and field size
#define GCR_CONTROL 0x0010
#define SYNCDIS_SHIFT	5
#define SYNCDIS_BITS 1
#define MEM_FIFOS_1_1_DISABLE_SHIFT 6
#define MEM_FIFOS_1_1_DISABLE_BITS 1
#define MEM_UC_FORCE_PEND_RESP_SHIFT 20
#define MEM_UC_FORCE_PEND_RESP_BITS 1
#define MEM_UCA_FULLY_ORDERED_SHIFT 21
#define MEM_UCA_FULLY_ORDERED_BITS 1
#define MEM_UCA_UNBUFFERED_SHIFT 22
#define MEM_UCA_UNBUFFERED_BITS 1
#define MEM_UCA_UNBUFFERABLE_SHIFT 23
#define MEM_UCA_UNBUFFERABLE_BITS 1

// GCR Offset for GCR_REV, field positions and field size
#define GCR_REV 0x0030
#define MAJOR_REV_GCR_SHIFT 0
#define MAJOR_REV_GCR_BITS 8
#define MINOR_REV_GCR_SHIFT 8
#define MINOR_REV_GCR_BITS 8

//GCR Offsets for GCR_ERROR_CONTROL
#define GCR_ERROR_CONTROL 0x0038
#define L2_ECC_SUPPORTED_SHIFT 0
#define L2_ECC_SUPPORTED_BITS 1
#define L2_ECC_EN_SHIFT 1
#define L2_ECC_EN_BITS 1
#define MEM_PARITY_SUPPORTED_SHIFT 4
#define MEM_PARITY_SUPPORTED_BITS 1
#define MEM_PARITY_EN_SHIFT 5
#define MEM_PARITY_EN_BITS_SHIFT 1


// CM3 Error Types encoding
#define MP_CORRECTABLE_ECC_ERR 1
#define MP_REQUEST_DECODE_ERR 2
#define MP_UNCORRECTABLE_ECC_ERR 3
#define MP_PARITY_ERR 4
#define MP_COHERENCE_ERR 5
#define CMBIU_REQUEST_DECODE_ERR 6
#define CMBIU_PARITY_ERR 7
#define CMBIU_AXI_RESP_ERR 8
#define RBI_BUS_ERR 10

// GCR Offset for GCR_ERROR_MASK
#define GCR_ERROR_MASK 0x0040

// GCR Offset for GCR_ERROR_CAUSE, field positions and field size
#define GCR_ERROR_CAUSE 0x0048
#define CM3_ERROR_TYPE_SHIFT 58
#define CM_ERROR_TYPE_BITS 6
#define CM_ERROR_INFO_SHIFT 0
#define CM_ERROR_INFO_BITS 58

// GCR Offset for GCR_ERROR_ADDR
#define GCR_ERROR_ADDR 0x0050
#define CM3_ERROR_ADDR_SHIFT 0
#define CM3_ERROR_ADDR_BITS 48


// GCR Offset for GCR_ERROR_MULT
#define GCR_ERROR_MULT 0x0058
#define CM_ERROR_2ND_SHIFT 0
#define CM_ERROR_2ND_BITS 5

// GCR Offsets for GCR_CUSTOM_BASE
#define CUSTOM_BASE_REGISTER 0x0060
#define GGU_EN_SHIFT 0
#define GGU_EN_BITS 1
#define CUSTOM_BASE_SHIFT 16
#define CUSTOM_BASE_BITS 32

// GCR Offsets for GCR_CUSTOM_STATUS
#define GCR_CUSTOM_STATUS 0x0068
#define GGU_EX_SHIFT 0
#define GGU_EX_BITS 1

// GCR Offset for GCR_GIC_BASE, fields and sizes
#define GCR_GIC_BASE 0x0080
#define GIC_BASEADDRESS_SHIFT 17
#define GIC_BASEADDRESS_BITS 31
#define GIC_EN_SHIFT 0
#define GIC_EN_BITS 1

// GCR Offset for GCR_CPC_BAS, fields and sizes
#define GCR_CPC_BASE 0x0088
#define CPC_BASEADDRESS_SHIFT 15
#define CPC_BASEADDRESS_BITS 33
#define CPC_EN_SHIFT 0
#define CPC_EN_BITS 1

// GCR Offset for GCR_GIC_STATUS, fields and sizes
#define GCR_GIC_STATUS 0x00D0
#define GIC_EX_SHIFT 0
#define GIC_EX_BITS 1

// GCR Offset for GCR_CACHE_REV, fields and sizes
#define GCR_CACHE_REV 0x00E0
#define MAJOR_REV_CACHE_SHIFT 0
#define MAJOR_REV_CACHE_BITS 8
#define MINOR_REV_CACHE_SHIFT 7
#define MINOR_REV_CACHE_BITS 8

// GCR Offset for GCR_CPC_STATUS, fields and sizes
#define GCR_CPC_STATUS 0x00F0
#define CPC_EX_SHIFT 0
#define CPC_EX_BITS 1

// GCR offsets for GCR_IOC_BASE
#define GCR_IOC_BASE 0x0100
#define IOC_REG_EN_SHIFT 0
#define IOC_REG_EN_BITS 1


// GCR offsets for GCR_IOMMU_STATUS, fields and sizes
#define GCR_IOMMU_STATUS 0x0108
#define IOMMU1_SHIFT 0
#define IOMMU1_BITS 1
#define IOMMU0_SHIFT 2
#define IOMMU0_BITS 1

// GCR offsets for GCR_ACCESS
#define GCR_ACCESS 0x0120
#define CORES_SHIFT 0
#define CORES_BITS 6
#define IOCUS_SHIFT 16
#define IOCUS_BITS 2

// GCR offsets for GCR_L2_CONFIG
#define GCR_L2_CONFIG 0x0130
#define L2_ASSOC_SHIFT 0
#define L2_ASSOC_BITS 8
#define L2_LINE_SIZE_SHIFT 8
#define L2_LINE_SIZE_BITS 4
#define L2_SET_SIZE_SHIFT 12
#define L2_SET_SIZE_BITS 4
#define L2_BYPASS_SHIFT 20
#define L2_BYPASS_BITS 1
#define COP_DATA_ECC_WE_SHIFT 24
#define COP_DATA_ECC_WE_BITS 1
#define COP_TAG_ECC_WE_SHIFT 25
#define COP_TAG_ECC_WE_BITS 1
#define COP_LRU_WE_SHIFT 26
#define COP_LRU_WE_BITS 1
#define L2_REG_EXISTS_SHIFT 31
#define L2_REG_EXISTS_BITS 1

// GCR Offset for GCR_IOCU1_REV, fields and sizes
#define GCR_IOCU1_REV 0x0200
#define MAJOR_REV_IOCU_SHIFT 0
#define MAJOR_REV_IOCU_BITS 8
#define MINOR_REV_IOCU_SHIFT 7
#define MINOR_REV_IOCU_BITS 8

// GCR Offset for GCR_L2_RAM_CONFIG
#define GCR_L2_RAM_CONFIG 0x0240
#define L2_DATARAM_STALLS_SHIFT 0
#define L2_DATARAM_STALLS_BITS 2
#define L2_WSRAM_STALLS_SHIFT 4
#define L2_WSRAM_STALLS_BITS 2
#define L2_TAGRAM_STALLS_SHIFT 8
#define L2_TAGRAM_STALLS_BITS 2
#define HCI_SUPPORTED_SHIFT 29
#define HCI_SUPPORTED_BITS 1
#define HCI_DONE_SHIFT 30
#define HCI_DONE_BITS 1

// GCR Offset for GCR_L2_TAG_ADDR
#define GCR_L2_TAG_ADDR 0x0600
#define TAG_ADDR_SHIFT 0
#define TAG_ADDR_BITS 48

// GCR Offset for GCR_L2_TAG_STATE
#define GCR_L2_TAG_STATE 0x0608
#define TAG_STATE_SHIFT 0
#define TAG_STATE_BITS 13
#define TAG_LRU_SHIFT 32
#define TAG_LRU_BITS 15

// GCR Offset for GCR_L2_DATA
#define GCR_L2_DATA 0x0610
// one field data covers the full 64 bits

// GCR Offset for GCR_L2_ECC
#define GCR_L2_ECC 0x0618
#define DATA_ECC_SHIFT 0
#define DATA_ECC_BITS 8
#define TAG_ECC_SHIFT 32
#define TAG_ECC_BITS 8

// GCR L2 Cache Op State machine NOTE only for CM 3.5
#define GCR_L2SM_COP 0x0620
#define L2SM_COP_REG_PRESENT_BITS 1
#define L2SM_COP_REG_PRESENT_SHIFT 31

#define L2SM_COP_RESULT_BITS 3
#define L2SM_COP_RESULT_SHIFT 6
#define  L2SM_COP_RESULT_DONT_CARE 0x0
#define  L2SM_COP_RESULT_DONE_NO_ERR 0x1
#define  L2SM_COP_RESULT_DONE_ERR 0x2
#define  L2SM_COP_RESULT_ABORT_NO_ERR 0x3
#define  L2SM_COP_RESULT_ABORT_ERR 0x4

#define L2SM_COP_MODE_BITS 1
#define L2SM_COP_MODE_SHIFT 5
#define  L2SM_COP_MODE_IDLE 0x0
#define  L2SM_COP_MODE_RUNNING

#define L2SM_COP_TYPE_BITS 3
#define L2SM_COP_TYPE_SHIFT 2
#define  L2SM_COP_TYPE_INDEX_WB_INV 0x1
#define  L2SM_COP_TYPE_INDEX_STORE_TAG_RAM 0x1
#define  L2SM_COP_TYPE_INDEX_STORE_TAG_DATA_RAM 0x2
#define  L2SM_COP_TYPE_HIT_INV 0x4
#define  L2SM_COP_TYPE_HIT_WB_INV 0x5
#define  L2SM_COP_TYPE_HIT_WB 0x6
#define  L2SM_COP_TYPE_FETCH_LOCK 0x7

#define L2SM_COP_CMD_BITS 2
#define L2SM_COP_CMD_SHIFT 0
#define  L2SM_COP_CMD_NOP 0x0
#define  L2SM_COP_CMD_START 0x1
#define  L2SM_COP_CMD_ABORT 0x3


// GCR Core Local and Core other offsets
#define CORE_LOCAL_CONTROL_BLOCK 0x2000
#define CORE_OTHER_CONTROL_BLOCK 0x4000

// GCR Core Local and Other COHERENCE, fields and sizes
#define GCR_CL_COH_EN 0x0008
#define GCR_CO_COH_EN 0x0008
#define COH_EN_SHIFT 0
#define COH_EN_BITS 1

// GCR Core Local and Other CONFIG , fields and sizes
#define GCR_CL_CONFIG 0x0010
#define GCR_CO_CONFIG 0x0010
#define IOCU_TYPE_SHIFT 10
#define IOCU_TYPE_BITS 2
#define PVPE_SHIFT 0
#define PVPE_BITS 10

// GCR local and other REDIRECT NOTE only for CM 3.5
#define GCR_CL_REDIRECT 0x0018
#define GCR_CO_REDIRECT 0x0018
#define CLUSTER_REDIRECT_EN_BITS 1
#define CLUSTER_REDIRECT_EN_SHIFT 31
#define GIC_REDIRECT_EN_BITS 1
#define GIC_REDIRECT_EN_SHIFT 30
#define BLOCK_REDIRECT_BITS 2
#define BLOCK_REDIRECT_SHIFT 24
#define CLUSTER_REDIRECT_BITS 6
#define CLUSTER_REDIRECT_SHIFT 16
#define CORE_REDIRECT_BITS 6
#define CORE_REDIRECT_SHIFT 8
#define VP_REDIRECT_BITS 3
#define VP_REDIRECT_SHIFT 0

// GCR Core Local and Other OTHER, fields and sizes NOTE only for CM 3
#define GCR_CL_OTHER 0x0018
#define GCR_CO_OTHER 0x0018
#define VP_OTHER_NUM_SHIFT 0
#define VP_OTHER_NUM_BITS 3
#define CORE_OTHER_SHIFT 8
#define CORE_OTHER_BITS 6
#define CLUSTER_OTHER_SHIFT 16
#define CLUSTER_OTHER_BITS 1

// GCR Core Local and Other RESET_BASE, fields and sizes
#define GCR_CL_RESET_BASE 0x0020
#define GCR_CO_RESET_BASE 0x0020
#define SELECT_BEV_SHIFT 0
#define SELECT_BEV_BITS 1
#define EXCBASE_SHIFT 12
#define EXCBASE_BITS 20


// GCR Core Local and Other ID
#define GCR_CL_ID 0x0028
#define GCR_CO_ID 0x0028
#define CORENUM_SHIFT 0
#define CORENUM_BITS 8
#define CLUSTER_ID_SHIFT 8
#define CLUSTER_ID_BITS 8

// GCR Global Debug Block Offsets
#define Global_Debug_Block 0x6000

// GCR Global Debug GCR_DB_TCBCONTROLB, fields and sizes
#define GCR_DB_TCBCONTROLB 0x0008
#define WE_DB_TCBCONTROLB_SHIFT 31
#define WE_DB_TCBCONTROLB_BITS 1
#define TWSRC_WIDTH_SHIFT 26
#define TWSRC_WIDTH_BITS 2
#define TRPAD_SHIFT 18
#define TRPAD_BITS 1
#define RM_SHIFT 16
#define RM_BITS 1
#define TR_SHIFT 15
#define TR_BITS 1
#define BF_SHIFT 14
#define BF_BITS 1
#define TM_SHIFT 12
#define TM_BITS 2
#define CR_SHIFT 8
#define CR_BITS 3
#define CAL_SHIFT 7
#define CAL_BITS 1
#define OFC_SHIFT 1
#define OFC_BITS 1
#define FUNNEL_TRACE_ENABLE_SHIFT 0
#define FUNNEL_TRACE_ENABLE_BITS 1

// GCR Global Debug GCR_DB_TCBCONTROLD, fields and sizes
#define GCR_DB_TCBCONTROLD 0x0010
#define P4_CTL_SHIFT 24
#define P4_CTL_BITS 2
#define P3_CTL_SHIFT 22
#define P3_CTL_BITS 2
#define P2_CTL_SHIFT 20
#define P2_CTL_BITS 2
#define P1_CTL_SHIFT 18
#define P1_CTL_BITS 2
#define P0_CTL_SHIFT 16
#define P0_CTL_BITS 2
#define TW_SRC_VAL_SHIFT 8
#define TW_SRC_VAL_BITS 3
#define TRACE_WB_SHIFT 7
#define TRACE_WB_BITS 1
#define CM_INHIBIT_OVERFLOW_SHIFT 5
#define CM_INHIBIT_OVERFLOW_BITS 1
#define TLEV_SHIFT 3
#define TLEV_BITS 2
#define AE_PER_PORT_SHIFT 2
#define AE_PER_PORT_BITS 1
#define GLOBAL_CM_EN_SHIFT 1
#define GLOBAL_CM_EN_BITS 1
#define CM_EN_SHIFT 0
#define CM_EN_BITS 1

// GCR Global Debug GCR_DB_TCBCONTROLE, fields and sizes
#define GCR_DB_TCBCONTROLE 0x0020
#define TrIdle_SHIFT 8
#define TrIdle_BITS 1
#define PeC_SHIFT 0
#define PeC_BITS 1

// GCR Global Debug GCR_DB_TCBConfig, fields and sizes
#define GCR_DB_TCBCONFIG 0x0028
#define CF1_SHIFT 31
#define CF1_BITS 1
#define SZ_SHIFT 17
#define SZ_BITS_SHIFT 4
#define CRMAX_SHIFT 14
#define CRMAX_BITS 3
#define CRMIN_SHIFT 11
#define CRMIN_BITS 3
#define PW_SHIFT 9
#define PW_BITS 2
#define ONT_SHIFT 5
#define ONT_BITS 1
#define OFT_SHIFT 4
#define OFT_BITS 1
#define TCB_REV_SHIFT 0
#define TCB_REV_BITS 4

// GCR Global Debug GCR_DB_PC_CTL, fields and sizes
#define GCR_DB_PC_CTL 0x0100
#define PERF_INT_EN_SHIFT 30
#define PERF_INT_EN_BITS 1
#define PERF_OVF_STOP_SHIFT 29
#define PERF_OVF_STOP_BITS 1
#define P1_RESET_SHIFT 9
#define P1_RESET_BITS 1
#define P1_COUNT_ON_SHIFT 8
#define P1_COUNT_ON_BITS 1
#define P0_RESET_SHIFT 7
#define P0_RESET_BITS 1
#define P0_COUNT_ON_SHIFT 6
#define P0_COUNT_ON_BITS 1
#define CYCL_CNT_RESET_SHIFT 5
#define CYCL_CNT_RESET_BITS 1
#define CYCL_CNT__ON_SHIFT 4
#define CYCL_CNT__ON_BITS 1
#define PERF_NUM_CNT_SHIFT 0
#define PERF_NUM_CNT_BITS 4

// GCR Global Debug Read Pointer GCR_DB_TCBRDP
#define GCR_DB_TCBRDP 0x0108

// GCR Global Debug Write Pointer GCR_DB_TCBWDP
#define GCR_DB_TCBWRP 0x0110

// GCR Global Debug Start Pointer GCR_DB_TCBSTP
#define GCR_DB_TCBSTP 0x0118

// GCR_DB_PC_OV, fields and sizes
#define GCR_DB_PC_OV 0x0120
#define P1_OVERFLOW_SHIFT 2
#define P1_OVERFLOW_BITS 1
#define P0_OVERFLOW_SHIFT 1
#define P0_OVERFLOW_BITS 1_BITS 1
#define CYCL_CNT_OVERFLOW_SHIFT 0
#define CYCL_CNT_OVERFLOW_BITS 1

// GCR Global Debug GCR_DB_PC_EVENT, fields and sizes
#define GCR_DB_PC_EVENT 0x0130
#define P1_EVENT_SHIFT 8
#define P1_EVENT_BITS 8
#define P0_EVENT_SHIFT 0
#define P0_EVENT_BITS 8

// GCR Global Debug GCR_DB_PC_CYCLE
#define GCR_DB_PC_CYCLE 0x0180


// GCR Global Debug Qualifier and count registers
#define GCR_DB_PC_QUAL0 0x0190
#define GCR_DB_PC_CNT0 0x0198
#define GCR_DB_PC_QUAL1 0x01a0
#define GCR_DB_PC_CNT1 0x01a8

// GCR Global Debug Trace word access registers
#define GCR_DB_TCBTW_LO 0x0200
#define GCR_DB_TCBTW_HI 0x0208

// GIC Offsets within the Global interrupt controller

#define GIC_SH_CONFIG 0x0000
#define VZP_SHIFT 31
#define VZP_BITS 1
#define VZE_SHIFT 30
#define VZE_BITS 1
#define IRC_SHIFT 29
#define IRC_BITS 1
#define COUNTSTOP_SHIFT 28
#define COUNTSTOP_BITS 1
#define COUNTBITS_SHIFT 24
#define COUNTBITS_BITS 4
#define NUMINTERRUPTS_SHIFT 16
#define NUMINTERRUPTS_BITS 8
#define IRGID_SHIFT 8
#define IRGID_BITS 8
#define PVPES_SHIFT 0
#define PVPES_BITS 7

// GIC Offset for GIC_SH_Counter
#define GIC_SH_Counter 0x0010
// 64 bits of count

// GIC Offset for GIC_RevisionID
#define GIC_RevisionID 0x0020
#define GIC_MINOR_REV_SHIFT 0
#define GIC_MINOR_REV_BITS 8
#define GIC_MAJOR_REV_SHIFT 8
#define GIC_MAJOR_REV_BITS 8

// GIC Offset for GIC_SH_INT_AVAIL
#define GIC_SH_INT_AVAIL63_0 0x0028
#define GIC_SH_INT_AVAIL127_64 0x0030
#define GIC_SH_INT_AVAIL191_128 0x0038
#define GIC_SH_INT_AVAIL255_192 0x0040

// GIC Offset for GIC_SH_GID_CONFIG
#define GIC_SH_GID_CONFIG63_0 0x0080
#define GIC_SH_GID_CONFIG127_64 0x0088
#define GIC_SH_GID_CONFIG191_128 0x0090
#define GIC_SH_GID_CONFIG255_192 0x0098


// GIC Offset for GIC_SH_POL
#define GIC_SH_POL63_0 0x0100
#define GIC_SH_POL127_64 0x0108
#define GIC_SH_POL191_128 0x0110
#define GIC_SH_POL255_192 0x0118

// GIC Offset for
#define GIC_SH_TRIG63_0 0x0180
#define GIC_SH_TRIG127_64 0x0188
#define GIC_SH_TRIG191_128 0x0190
#define GIC_SH_TRIG255_192 0x0198

#define GIC_SH_DUAL63_0 0x0200
#define GIC_SH_DUAL127_64 0x0208
#define GIC_SH_DUAL191_128 0x0210
#define GIC_SH_DUAL255_192 0x0218

#define GIC_SH_WEDGE 0x0280

#define GIC_SH_RMASK63_0 0x0300
#define GIC_SH_RMASK127_64 0x0308
#define GIC_SH_RMASK191_128 0x0310
#define GIC_SH_RMASK255_192 0x0318

#define GIC_SH_SMASK63_00 0x0380
#define GIC_SH_SMASK127_64 0x0384
#define GIC_SH_SMASK191_128 0x0390
#define GIC_SH_SMASK255_192 0x0398

#define GIC_SH_MASK63_00 0x0400
#define GIC_SH_MASK127_64 0x0408
#define GIC_SH_MASK191_128 0x0410
#define GIC_SH_MASK255_192 0x0418

#define GIC_SH_PEND63_00 0x0480
#define GIC_SH_PEND127_64 0x0488
#define GIC_SH_PEND191_128 0x0490
#define GIC_SH_PEND255_192 0x0498

// Global MAP to Pin GIC_SH_MAP_PIN + (4 x interrupt_source)
#define GIC_SH_MAP_PIN 0x0500

#define GIC_SH_MAP_SPACER 0x20

// Map source to VPs 63 - 0 GIC_SH_MAP0_VP63_0 + (0x20 x interrupt source)
#define GIC_SH_MAP0_VP63_0 0x2000

// Map source to core 63 - 0 GIC_SH_MAP0_CORE31_0 + (0x20 x interrupt source)
#define GIC_SH_MAP0_CORE63_0 0x2000


#define GIC_VB_DINT_SEND 0x6000

// GIC VPE Local offsets (note the VPEL)
#define GIC_VPE_LOCAL_SECTION_OFFSET 0x8000

#define GIC_VPEL_CTL 0x0000
#define GIC_VPEL_PEND 0x0004
#define GIC_VPEL_MASK 0x0008
#define GIC_VPEL_RMASK 0x000c
#define GIC_VPEL_SMASK 0x0010
#define GIC_VPEL_WD_MAP 0x0040
#define GIC_VPEL_COMPARE_MAP 0x0044
#define GIC_VPEL_TIMER_MAP (0x0048 )
#define GIC_VPEL_FDC_MAP 0x004c
#define GIC_VPEL_PERFCTR_MAP 0x0050
#define GIC_VPEL_SWInt0_MAP 0x0054
#define GIC_VPEL_SWInt1_MAP 0x0058
#define GIC_VPEL_OTHER_ADDR 0x0080
#define GIC_VPEL_IDENT 0x0088
#define GIC_VPEL_WD_CONFIG0 0x0090
#define GIC_VPEL_WD_COUNT0 0x0094
#define GIC_VPEL_WD_INITIAL0 0x0098
#define GIC_VPEL_CompareLo 0x00A0
#define GIC_VPEL_CompareHi 0x00A4

// NOTE: EIC Shadow set GIC_VPEL_EICSS + (4 x interrupt number)
#define GIC_VPEL_EICSS 0x0100
#define GIC_VL_DINT_PART 0x3000
#define GIC_VL_BRK_GROUP 0x3080

// GIC VPE Other offsets (note the VPEO)
#define GIC_VPE_OTHER_SECTION_OFFSET 0xc000

#define GIC_VPEO_CTL 0x0000
#define GIC_VPEO_PEND 0x0004
#define GIC_VPEO_MASK 0x0008
#define GIC_VPEO_RMASK 0x000c
#define GIC_VPEO_SMASK 0x0010
#define GIC_VPEO_WD_MAP 0x0040
#define GIC_VPEO_COMPARE_MAP 0x0044
#define GIC_VPEO_TIMER_MAP 0x0048
#define GIC_VPEO_FDC_MAP 0x004c
#define GIC_VPEO_PERFCTR_MAP 0x0050
#define GIC_VPEO_SWInt0_MAP 0x0054
#define GIC_VPEO_SWInt1_MAP 0x0058
#define GIC_VPEO_OTHER_ADDR 0x0080
#define GIC_VPEO_IDENT 0x0088
#define GIC_VPEO_WD_CONFIG0 0x0090
#define GIC_VPEO_WD_COUNT0 0x0094
#define GIC_VPEO_WD_INITIAL0 0x0098
#define GIC_VPEO_CompareLo 0x00A0
#define GIC_VPEO_CompareHi 0x00A4

// NOTE: EIC Shadow set GIC_VPEO_EICSS + (4 x interrupt number)
#define GIC_VPEO_EICSS 0x0100
#define GIC_VO_DINT_PART 0x3000
#define GIC_VO_BRK_GROUP 0x3080

// GIC CORE Local offsets (note the COREL)
#define GIC_CORE_LOCAL_SECTION_OFFSET 0x8000

#define GIC_COREL_CTL 0x0000
#define GIC_COREL_PEND 0x0004
#define GIC_COREL_MASK 0x0008
#define GIC_COREL_RMASK 0x000c
#define GIC_COREL_SMASK 0x0010
#define GIC_COREL_WD_MAP 0x0040
#define GIC_COREL_COMPARE_MAP 0x0044
#define GIC_COREL_TIMER_MAP 0x0048
#define GIC_COREL_FDC_MAP 0x004c
#define GIC_COREL_PERFCTR_MAP 0x0050
#define GIC_COREL_SWInt0_MAP 0x0054
#define GIC_COREL_SWInt1_MAP 0x0058
#define GIC_COREL_OTHER_ADDR 0x0080
#define GIC_COREL_IDENT 0x0088
#define GIC_COREL_WD_CONFIG0 0x0090
#define GIC_COREL_WD_COUNT0 0x0094
#define GIC_COREL_WD_INITIAL0 0x0098
#define GIC_COREL_CompareLo 0x00A0
#define GIC_COREL_CompareHi 0x00A4
// NOTE: EIC Shadow set GIC_COREL_EICSS + (4 x interrupt number)
#define GIC_COREL_EICSS 0x0100
#define GIC_COREL_DINT_PART 0x3000
#define GIC_COREL_BRK_GROUP 0x3080

// GIC CORE Other offsets (note the COREO)
#define GIC_CORE_OTHER_SECTION_OFFSET 0xc000
#define GIC_COREO_CTL 0x0000
#define GIC_COREO_PEND 0x0004
#define GIC_COREO_MASK 0x0008
#define GIC_COREO_RMASK 0x000c
#define GIC_COREO_SMASK 0x0010
#define GIC_COREO_WD_MAP 0x0040
#define GIC_COREO_COMPARE_MAP 0x0044
#define GIC_COREO_TIMER_MAP 0x0048
#define GIC_COREO_FDC_MAP 0x004c
#define GIC_COREO_PERFCTR_MAP 0x0050
#define GIC_COREO_SWInt0_MAP 0x0054
#define GIC_COREO_SWInt1_MAP 0x0058
#define GIC_COREO_OTHER_ADDR 0x0080
#define GIC_COREO_IDENT 0x0088
#define GIC_COREO_WD_CONFIG0 0x0090
#define GIC_COREO_WD_COUNT0 0x0094
#define GIC_COREO_WD_INITIAL0 0x0098
#define GIC_COREO_CompareLo 0x00A0
#define GIC_COREO_CompareHi 0x00A4
// NOTE: EIC Shadow set GIC_COREO_EICSS + (4 x interrupt number)
#define GIC_COREO_EICSS 0x0100
#define GIC_COREO_DINT_PART 0x3000
#define GIC_COREO_BRK_GROUP 0x3080


// Bit fields for Local Interrupt Control Register (GIC_COREi_CTL) or for MT (GIC_VPEi_CTL) or
#define FDC_ROUTABLE_SHIFT 4
#define FDC_ROUTABLE_BITS 1
#define SWINT_ROUTABLE_SHIFT 3
#define SWINT_ROUTABLE_BITS 1
#define PERFCOUNT_ROUTABLE_SHIFT 2
#define PERFCOUNT_ROUTABLE_BITS 1
#define TIMER_ROUTABLE_SHIFT 1
#define TIMER_ROUTABLE_BITS 1
#define EIC_MODE_SHIFT 0
#define EIC_MODE_BITS 1

// Bit  fields for Local Interrupt Pending Registers (GIC_COREi_PEND) or for MT (GIC_VPEi_PEND)
#define FDC_PEND_SHIFT 6
#define FDC_PEND_BITS 1
#define SWINT1_PEND_SHIFT 5
#define SWINT1_PEND_BITS 1
#define SWINT0_PEND_SHIFT 4
#define SWINT0_PEND_BITS 1
#define PERFCOUNT_PEND_SHIFT 3
#define PERFCOUNT_PEND_BITS 1
#define TIMER_PEND_SHIFT 2
#define TIMER_PEND_BITS 1
#define COMPARE_PEND_SHIFT 1
#define COMPARE_PEND_BITS 1
#define WD_PEND_SHIFT 0
#define WD_PEND_BITS 1

// Bit fields for Local Interrupt Mask Registers (GIC_COREi_MASK) or for MT (GIC_VPEi_MASK)
#define FDC_MASK_SHIFT 6
#define FDC_MASK_BITS 1
#define SWINT1_MASK_SHIFT 5
#define SWINT1_MASK_BITS 1
#define SWINT0_MASK_SHIFT 4
#define SWINT0_MASK_BITS 1
#define PERFCOUNT_MASK_SHIFT 3
#define PERFCOUNT_MASK_BITS 1
#define TIMER_MASK_SHIFT 2
#define TIMER_MASK_BITS 1
#define COMPARE_MASK_SHIFT 1
#define COMPARE_MASK_BITS 1
#define WD_MASK_SHIFT 0
#define WD_MASK_BITS 1

// Bit fields for Local Interrupt Reset Mask Registers (GIC_COREi_RMASK) or for MT (GIC_VPEi_RMASK)
#define FDC_MASK_RESET_SHIFT 6
#define FDC_MASK_RESET_BITS 1
#define SWINT1_MASK_RESET_SHIFT 5
#define SWINT1_MASK_RESET_BITS 1
#define SWINT0_MASK_RESET_SHIFT 4
#define SWINT0_MASK_RESET_BITS 1
#define PERFCOUNT_MASK_RESET_SHIFT 3
#define PERFCOUNT_MASK_RESET_BITS 1
#define TIMER_MASK_RESET_SHIFT 2
#define TIMER_MASK_RESET_BITS 1
#define COMPARE_MASK_RESET_SHIFT 1
#define COMPARE_MASK_RESET_BITS 1
#define WD_MASK_RESET_SHIFT 0
#define WD_MASK_RESET_BITS 1

// Bit fields for Local Interrupt Set Mask Registers (GIC_COREi_SMASK) or for MT (GIC_VPEi_SMASK)
#define FDC_MASK_SET_SHIFT 6
#define FDC_MASK_SET_BITS 1
#define SWINT1_MASK_SET_SHIFT 5
#define SWINT1_MASK_SET_BITS 1
#define SWINT0_MASK_SET_SHIFT 4
#define SWINT0_MASK_SET_BITS 1
#define PERFCOUNT_MASK_SET_SHIFT 3
#define PERFCOUNT_MASK_SET_BITS 1
#define TIMER_MASK_SET_SHIFT 2
#define TIMER_MASK_SET_BITS 1
#define COMPARE_MASK_SET_SHIFT 1
#define COMPARE_MASK_SET_BITS 1
#define WD_MASK_SET_SHIFT 0
#define WD_MASK_SET_BITS 1

// Bit fields for CORE-Other or for MT VPE-Other Addressing Register
#define VPENum_SHIFT 0
#define VPENum_BITS 16
#define CORENum_SHIFT 0
#define CORENum_BITS 16

// Bit fields for Core-Local Identification Register (GIC_COREi_IDENT) or for MT (GIC_VPEi_IDENT)
#define VPENumIDENT_SHIFT 0
#define VPENumIDENT_BITS 32
#define CORENumIDENT_SHIFT 0
#define CORENumIDENT_BITS 32

// Bit fields for Local EIC Shadow Set Registers (GIC_COREi_EICSSj) or for MT (GIC_VPEi_EICSSj)
#define EIC_SS_SHIFT 0
#define EIC_SS_BITS 4

// Bit fields for Local WatchDog/Compare/PerfCount/SWIntx Map to Pin Registers
#define MAP_TO_PIN_SHIFT 31
#define MAP_TO_PIN_BITS 1
#define MAP_TO_NMI_SHIFT 30
#define MAP_TO_NMI_BITS 1
#define MAP_TO_YQ_SHIFT 29
#define MAP_TO_YQ_BITS 1
#define MAP_SHIFT 0
#define MAP_BITS 6

// Bit fields for Watchdog Timer Config Register (GIC_COREi_WD_CONFIGk) or for MT (GIC_VPEi_WD_CONFIGk)
#define WDRESET_SHIFT 7
#define WDRESET_BITS 1
#define WDINTR_SHIFT 6
#define WDINTR_BITS 1
#define WAITMODE_CNTRL_SHIFT 5
#define WAITMODE_CNTRL_BITS 1
#define DEBUGMODE_CNTRL_SHIFT 5
#define DEBUGMODE_CNTRL_BITS 1
#define TYPE_SHIFT 1
#define TYPE_BITS 3
// TYPE Filed encoding:
#define WD_One_Trip_Mode 0
#define WD_Second_Countdown_Mode 1
#define PIT_Mode 2

// Bit fields for Local DINT Group Participate Register (GIC_Cx_DINT_PART) or for MT (GIC_Vx_DINT_PART)
#define DINT_Group_Particpate_SHIFT 0
#define DINT_Group_Particpate_BITS 1

// GIC, GIC User Mode Visible Section Offsets
#define USER_MODE_VISIBLE_SECTION_OFFSET 0x10000
#define GIC_SH_COUNTERLO 0x0000
#define GIC_SH_COUNTERHI 0x0004

// Cluster Power Controller Global Section
// CPC Block CPC_ACCESS_REG, fields and sizes
#define CPC_ACCESS_REG 0x000
#define CM_ACCESS_EN_SHIFT 0
#define CM_ACCESS_EN_BITS 8

// CPC Block CPC_SEQDEL_REG, fields and sizes
#define CPC_SEQDEL_REG 0x008
#define MICROSTEP_SHIFT 0
#define MICROSTEP_BITS 10

// CPC Block CPC_RAIL_REG, fields and sizes
#define CPC_RAIL_REG 0x010
#define RAILDELAY_SHIFT 0
#define RAILDELAY_BITS 10

// CPC Block CPC_RESETLEN_REG, fields and sizes
#define CPC_RESETLEN_REG 0x018
#define RESETLEN_SHIFT 0
#define RESETLEN_BITS 10

// CPC Block CPC_REVISION_REG, fields and sizes
#define CPC_REVISION_REG 0x020
#define MAJOR_REV_CPC_SHIFT 8
#define MAJOR_REV_CPC_BITS 8
#define MINOR_REV_CPC_SHIFT 0
#define MINOR_REV_CPC_BITS 8

// Cluster Power Controller Local and Other section
#define CPS_CORE_LOCAL_CONTROL_BLOCK 0x2000
#define CPS_CORE_OTHER_CONTROL_BLOCK 0x4000

// CPC Local and Other CPC_CMD_REG, command encoding
#define CPC_CMDL_REG 0x000
#define CPC_CMDO_REG 0x000
#define CLOCK_OFF 1
#define PWR_DOWN 2
#define PWR_UP 3
#define CPC_RESET 4

// CPC Local and Other CPC_STAT_CONF, fields, sizes and encodings
#define CPC_STATL_CONF_REG 0x008
#define CPC_STATO_CONF_REG 0x008
#define PWRUP_EVENT_SHIFT 23
#define PWRUP_EVENT_BITS 1
#define SEQ_STATE_SHIFT 19
#define SEQ_STATE_BITS 4

// sequencer state encodings
#define PWR_DOWN_STATE 0x0
#define VDD_OK_STATE 0x1
#define UP_DELAY_STATE 0x2
#define UCLK_OFF_STATE 0x3
#define CPC_RESET_STATE 0x4
#define CPC_RESET_DLY_STATE 0x5
#define NON_COHERENT_EXECUTION_STATE 0x6
#define COHERENT_EXECUTION_STATE 0x7
#define ISOLATE_STATE 0x8
#define CLR_BUS_STATE 0x8
#define DCLK_OFF_STATE 0xA

#define CLKGAT_IMPL_SHIFT 17
#define CLKGAT_IMPL_BITS 1
#define PWRDN_IMPL_SHIFT 16
#define PWRDN_IMPL_BITS 1
#define EJTAG_PROBE_SHIFT 15
#define EJTAG_PROBE_BITS 1
#define PWUP_POLICY_SHIFT 8
#define PWUP_POLICY_BITS 2
// Power up state encodings
#define POLICY_PWR_DOWN 0
#define POLICY_GO_CLOCK_OFF 1
#define POLICY_PWR_UP 2

#define IO_TRFFC_EN_SHIFT 4
#define IO_TRFFC_EN_BITS 1
#define CPC_CMD_STATE_SHIFT 0
#define CPC_CMD_STATE_BITS 4



// CPC VP Stop Register
#define CPC_CL_VP_STOP_REG 0x0020
#define CPC_CO_VP_STOP_REG 0x0020
#define VP_STOP_SHIFT 0
#define VP_STOP_BITS 4

// CPC VP Run Register
#define CPC_CL_VP_RUN_REG 0x0028
#define CPC_CO_VP_RUN_REG 0x0028
#define VP_RUN_SHIFT 0
#define VP_RUN_BITS 4

// CPC VP Running Register
#define CPC_CL_VP_RUNNING_REG 0x0030
#define CPC_CO_VP_RUNNING_REG 0x0030
#define VP_RUNNING_SHIFT 0
#define VP_RUNNING_BITS 4

#endif /* _CM3_H_ */
