/*
 * operating_transition.S
 */
 /*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <mips/asm.h>
#include <core_config.h>				// #define for GCR_CONFIG_ADDR
#include <mips/regdef.h>		// O32 GPR #defines
#include <mips/m32c0.h>			// CP0 register #defines
#include <cps.h>				// #defines for CM and CPC registers

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.
/**************************************************************************************
coherent_to_noncoherent()

must be executed on core that is transitioning
**************************************************************************************/
LEAF(coherent_to_noncoherent)

// 1. Switch to non-coherent CCA.

	mfc0	a2, C0_CONFIG	// read C0_Config
	li	    a3, 3			// CCA non-coherent
    ins	    a2, a3, 0, 3	// instert K0
	mtc0	a2, C0_CONFIG	// write C0_Config

// 2. Flush dirty data from data cache using IndexWritebackInvalidate CACHE instruction on all lines in the cache.

    la 		a2,     flush_dcache    // Flush dirty lines out of data cache
    jalr 	a2

// 3. If the instruction cache contains lines that are expected to be maintained by software as coherent (via globalized
// CACHE instructions), and the CPU is not going to go through a reset sequence, the instruction cache should be
// flushed using IndexInvalidate CACHE instructions.

// usually not the case (if it is call init_icache_ram())

// 4. Write GCR_CL_COHERENCE (Core Local GCR address 0x0008). Write 0 to all bits except bit for "self",
// which should stay set to 1. This is required so that the core can issue a coherent SYNC (step 6) to make sure all
// previous interventions are complete.

	// disable coherence from all other cores.
	// (Write access enabled via GCR_ACCESS by core 0.)
	mfc0    a0, C0_EBASE   	// Read CP0 EBase
	ext	    a1, a0, 0, 4  	// Extract CPUNum
	li	    a2, 0x01
	sllv	a3, a2, a1
	li      a0, GCR_CONFIG_ADDR
	sw	    a3, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE

// 5. Read GCR_CL_COHERENCE (ensures step 4 has completed).
//	the following loop makes sure the change has taken affect. The loop will exit if the
//  loop count is exceeded to insure the code is not in an endless loop or the change is verified.
	addu	v0, zero, zero
	addiu 	v1, zero, 101

1:	addiu 	v0, v0, 1
	bne		v0, v1, 2f
	// loop count reached return failure
	addiu 	v0, v0, -1
	jalr	zero, ra

2:	lw 		a2, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE

	bne 	a2, a3, 1b

// 6. Issue Coherent SYNC (intervention-only SYNC is fine).

	sync

// 7. Write 0 to GCR_CL_COHERENCE to completely remove core from coherence domain.

	sw	    zero, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE


// 8. Read GCR_CL_COHERENCE to ensure step 7 is complete.

2:	lw 		a2, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE
	bne 	a2, zero, 2b

    jalr	zero,      ra
END(coherent_to_noncoherent)

/**************************************************************************************
noncoherent_to_coherent()

must be executed on core that is transitioning
**************************************************************************************/
LEAF(noncoherent_to_coherent)

// 1. Caches must be initialized first (since last reset)

	// switch to non  cached mode
	mfc0	a2, C0_CONFIG	// read C0_Config
	li	    a3, 2			// CCA non-cache
    ins	    a2, a3, 0, 3	// instert K0
	mtc0	a2, C0_CONFIG	// write C0_Config

    la 		a2,     init_icache_ram    // Flush dirty lines out of data cache
    jalr 	a2

	// switch to non coherent cached mode
	mfc0	a2, C0_CONFIG	// read C0_Config
	li	    a3, 3			// CCA non-coherent
    ins	    a2, a3, 0, 3	// insert K0
	mtc0	a2, C0_CONFIG	// write C0_Config

    la 		a2, init_dcache_ram    // invalidate data cache
    jalr 	a2


// 2. There should be no data in the caches that will later be accessed coherently. Non-coherent data is treated as
// exclusive/modified which can lead to violations of the coherence protocol if other caches have copies of the data.


// 3 The GCR local coherence control register is programmed to add the core to the coherent domain.

	// enable coherence for this core.
	// (Write access enabled via GCR_ACCESS by core 0.)
	mfc0    a0, C0_EBASE   	// Read CP0 EBase
	ext	    a1, a0, 0, 4  	// Extract CPUNum
	li	    a2, 0x01
	sllv	a3, a2, a1
	li      a0, GCR_CONFIG_ADDR
	lw 		a2, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE
	or		a3, a3, a2 		// add core to coherency mask
	sw	    a3, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(a0)	// GCR_CL_COHERENCE
	ehb

// 4 Switch to coherent Cache Coherence Attribute (CCA).
	// switch to coherent cached mode
	mfc0	a2, C0_CONFIG	// read C0_Config
	li	    a3, 5			// CCA coherent
    ins	    a2, a3, 0, 3	// instert K0
	mtc0	a2, C0_CONFIG	// write C0_Config


// 5 Regular coherent programs can now start on this core.

    jalr	zero,      ra
END(noncoherent_to_coherent)
/**************************************************************************************

noncoherent_to_power_down(core number) should be executed by another core, (usually core 0)

a0 contains core number to be powered down
**************************************************************************************/

LEAF(noncoherent_to_power_down)
// 1. The GIC might need to be programmed to re-route interrupts away from this core.


// 2.  The CPC must be programmed to enter power-down mode.

	li      a1, CPC_BASE_ADDR			// Maintain address of CPC register block.

    // Send PwrDown command to core (argument A0) .
    move    a3, a0
    sll     a3, 16
    sw      a3, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(a1)
    li      a3, PWR_DOWN
    sw      a3, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(a1)

// 3. Core outputs are held inactive towards the CM. Completion of pending bus traffic is awaited and start of new
// traffic prevented using the SI_LPReq protocol.


// 4 The CPC initiates the clock and power shutdown micro-sequence.

    jalr	zero,      ra
END(noncoherent_to_power_down)
/**************************************************************************************
noncoherent_to_clock_off(core number) should be executed by another core, (usually core 0)

a0 contains core number to be put into Power Down state
**************************************************************************************/
LEAF(noncoherent_to_clock_off)

// 1. Programming a CPC CLOCK_OFF command will disable the clock tree root for this core.
	li      a1, CPC_BASE_ADDR			// Maintain address of CPC register block.

    // Send clock-off command to core (argument A0) causing execution at their reset exception vector.
    move    a3, a0
    sll     a3, 16
    sw      a3, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(a1)
    li      a3, CLOCK_OFF
    sw      a3, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(a1)

// 2. Core outputs are held inactive towards the CM. Completion of pending bus traffic is awaited and start of new
// traffic prevented using the SI_LPReq protocol.


// 3.The GIC might be programmed to re-route interrupts for this core to others.
    jalr	zero,      ra
END(noncoherent_to_clock_off)
/**************************************************************************************
clock_off_to_power_down(core number) should be executed by another core, (usually core 0)

a0 contains core number to be put into Power Down state
**************************************************************************************/
LEAF(clock_off_to_power_down)
// 1. The CPC must be programmed to enter power-off mode.

	li      a1, CPC_BASE_ADDR			// Maintain address of CPC register block.

    // Send PwrDown command to core (argument A0)
    move    a3, a0
    sll     a3, 16
    sw      a3, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(a1)
    li      a3, PWR_DOWN
    sw      a3, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(a1)

// 2. The CPC initiates the clock and power shutdown micro-sequence.

    jalr	zero,      ra
END(clock_off_to_power_down)
/**************************************************************************************
clock_off_to_noncoherent(core number) should be executed by another core, (usually core 0)

a0 contains core number to be transitioned from clock off to a coherent state
**************************************************************************************/
LEAF(clock_off_to_noncoherent)
// 1. The CPC command register is programmed to bring the core back on-line. A CPC_PwrUp command will let the
// core resume operation immediately, or, if a Reset command given, go through a reset sequence before becoming
// operational.

	li      a1, CPC_BASE_ADDR			// Maintain address of CPC register block.

    // Send PerUp command to core (argument A0) causing that core to enter non-chorent state.
    move    a3, a0
    sll     a3, 16
    sw      a3, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(a1)
    li      a3, PWR_UP
    // li      a3, CPC_RESET // use instead of PWR_UP if reboot is desired
    sw      a3, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(a1)

// 2. If the core bus was isolated due to earlier power modes, this isolation is removed.


// 3. The clock is applied and the core starts executing instructions.
    jalr	zero,      ra
END(clock_off_to_noncoherent)
/**************************************************************************************
power_down_to_noncoherent(core number) should be executed by another core, (usually core 0)

a0 contains core number to be transitioned from powerdown to a coherent state
**************************************************************************************/
LEAF(power_down_to_noncoherent)

// 1. The GCR local coherence control register must be set inactive for this core. Powering up into a coherent state
// with uninitialized caches may corrupt coherent data.

	li      a1, CPC_BASE_ADDR			// Maintain address of CPC register block.

    // Send PWR_UP command to core (argument A0) causing core to resume execution at the boot exception vector
    // 2. Software on another core can send a PwrUp or Reset command for this core or an SOC hardware signal can
	// request for the CPC to schedule a power-up sequence targeting non-coherent mode.
    move    a3, a0
    sll     a3, 16
    sw      a3, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(a1)
    li      a3, PWR_UP
    sw      a3, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(a1)

// 3. The CPC will schedule a power-up sequence and the core becomes operational outside the coherent domain.
// After the core becomes operational, execution continues at the boot vector provided while power-up mode reset.
// NOTE: reset is not automatically applied unless the core really was in the power-down state prior to a PwrUp
// command or hardware PwrUp signal.


// 4 The GIC might be reprogrammed to perform interrupt routing to this core.
    jalr	zero,      ra
END(power_down_to_noncoherent)
