/*
 * init_gic_CM26N.S
 *
 *  Initialization of the Global Interrupt Controller
*/
/*
Copyright (c) 2007-2020, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps.h>

/**************************************************************************************
**************************************************************************************/
LEAF(init_gic)

    beqz    r11_is_cps, done_gic					// Skip if non-CPS.

    la      a1, GCR_GIC_STATUS + GCR_CONFIG_ADDR	// Read GCR_GIC_STATUS
    lw      a0, 0(a1)
    ext     a0, a0, GIC_EX, GIC_EX_S				// Isolate  GCR_GIC_STATUS[GIC_EX].
    beqz    a0, done_gic							// If no gic then skip.

    bnez    r23_cpu_num, init_vpe_gic				// Only core0 vpe0 inits shared portion.

    li      a1, GCR_CONFIG_ADDR + GCR_GIC_BASE  	// Locate and enable GIC
    li      a0, GIC_P_BASE_ADDR | 1 				// Physical address + enable bit
    sw      a0, 0(a1)

#define	GIC_BASE	s1
    // Verify gic is 5 "slices" of 8 interrupts giving 40 interrupts.
    li      GIC_BASE, GIC_BASE_ADDR					// load GIC KSEG0 must be in uncached mode!
    lw      a0, GIC_SH_CONFIG(GIC_BASE)				// GIC_SH_CONFIG
    ext     a0, NUMINTERRUPTS, NUMINTERRUPTS_S		// NUMINTERRUPTS (actually slices - 1)
    li      a3, 4
    bge     a0, a3, configure_slices
    sdbbp   // Failed assertion that gic implements 40 external interrupts.

configure_slices:
	// This code supports up to 10 cores, with each core having up to 3 VPEs.
	// So we need as many as 30 inter-processor interrupts for these 30 VPEs
	// (processors).
	//
    // Hardcoded to set up the last 30 of 40 external interrupts (10..39) for IPI.
    // First set up interrupts 10...31 (top 22 bits of 31_0 registers)
    li      a0, 0xfffffc00
    sw      a0, GIC_SH_RMASK31_0(GIC_BASE)		// (disable		10..31)
    sw      a0, GIC_SH_POL31_0(GIC_BASE)		// (high/rise	10..31)
    sw      a0, GIC_SH_TRIG31_0(GIC_BASE)		// (edge		10..31)
    sw      a0, GIC_SH_SMASK31_00(GIC_BASE)		// (enable		10..31)

    // Then interrupts 32...40 (lower 8 bits of 63_32 registers)
    li      a0, 0xff
    sw      a0, GIC_SH_RMASK63_32(GIC_BASE)		// (disable		32..39)
    sw      a0, GIC_SH_POL63_32(GIC_BASE)		// (high/rise	32..39)
    sw      a0, GIC_SH_TRIG63_32(GIC_BASE)		// (edge		32..39)
    sw      a0, GIC_SH_SMASK63_32(GIC_BASE)		// (enable		32..39)

// Initialize configuration of shared interrupts

#define MAP_BIT_SET a0	// VPE bit for MAP Source to VPE register
#define CORE a1			// core within cluster
#define VPE a2			// VPE within core
#define SOURCE a3		// interrupt source number
#define	SOURCE_BASE t3	// interrupt source number base
#define	SLOTS	t0		// number of interrupt source slots per core
#define	SPACER	t1		// Space between Map source to VPE registers
#define BASE_OFFSET	t2	// offset index from the start of the GIC address region
#define	ONE	s0			// constant for VPE bit setting in MAP source to VPE register
#define	VPE_SLOTS	s2	// number of VPE slots per core

// initialize register starting values
	li		CORE, 0                     // Start with core 0
	li		SLOTS,	3
	li		SOURCE,	10
	li		SOURCE_BASE, 10             // Start mapping with interrupt #10
	li		SPACER, GIC_SH_MAP_SPACER
	li		ONE, 1
	li		VPE_SLOTS,	4               // For VPE numbering purposes, there are
                                        // 4 VPE "slots" per core.
CORE_LOOP:
	li		VPE,	0
VPE_LOOP:
// Calculate the VPE number.  This tells us which bit to set in GIC_SH_MAP_VP
// MAP_BIT_SET = (CORE * VPE_SLOTS) + (VPE)
	mul		MAP_BIT_SET, VPE_SLOTS, CORE					// compute the base bit index for the Core
	add		MAP_BIT_SET, MAP_BIT_SET, VPE					// add VPE number to the base bit index
	ext		MAP_BIT_SET, MAP_BIT_SET, 0, 5					// to handle VPE > 31, use only lower 5 bits
															// This value then works when VPE > 31 and
															// GIC_SH_MAP_VP_HI is used
// GIC Interrupt Map source to VPE
// build the Base offset from the GIC_BASE address for the Source register
	mul		BASE_OFFSET, SPACER, SOURCE						// compute the offset index for the particular Source register
	addi 	BASE_OFFSET, BASE_OFFSET, GIC_SH_MAP0_VPE31_0	// add base offset of the GIC Map register to get offset from the GIC_BASE address region
	li		t4, 8											// If core >= 8 then add 4 to BASE_OFFSET.  This bumps the
	blt		CORE, t4, 1f									// offset so that the GIC_SH_MAP_VP_HI register is used
	addiu	BASE_OFFSET, BASE_OFFSET, 4						// for cores 8 and 9 (where VPE > 31)
1:	sllv	MAP_BIT_SET, ONE, MAP_BIT_SET					// shift the bit for the particular VPE using the VPE bit index
	swx		MAP_BIT_SET, BASE_OFFSET(GIC_BASE)				// write the VPE bit to the MAP Source to VPE register

// increment the VPE and Source for next loop
	addi	VPE, VPE, 1
	addiu	SOURCE, SOURCE, 1
// check to see if the number of VPEs on core is exceeded if not goto the top of loop
// and program next source
	ble		VPE, r20_more_vpes, VPE_LOOP

// Once the number of VPE on core is exceeded then increment to the next Core
	addi	CORE, CORE, 1

// increment to next Source index
	mul		SOURCE, SLOTS, CORE								// starting slot index number for core
	add		SOURCE, SOURCE, SOURCE_BASE						// add the Source Base index

// check to see if the number of COREs on Cluster is exceeded if not goto the top
// of loop and program next source
	ble		CORE, r19_more_cores, CORE_LOOP

// Done programming Source to VPE for interprocessor interrupts

init_vpe_gic:
    // Initialize configuration of per Core or for MT vpe interrupts
    li      a1, (GIC_BASE_ADDR | GIC_CORE_LOCAL_SECTION_OFFSET)
    lw      a3, GIC_COREL_CTL(a1)

map_timer_int:
    ext     a0, a3, TIMER_ROUTABLE, TIMER_ROUTABLE_S
    beqz    a0, map_perfcount_int

    li      a0, 0x80000005               // Int5 is selected for timer routing
    sw      a0, GIC_COREL_TIMER_MAP(a1)

map_perfcount_int:
    ext     a0, a3, PERFCOUNT_ROUTABLE, PERFCOUNT_ROUTABLE_S
    beqz    a0, done_gic
    li      a0, 0x80000004               // Int4 is selected for performance routing
    sw      a0, GIC_COREL_PERFCTR_MAP(a1)

done_gic:
    jr      ra

#undef	MAP_BIT_SET
#undef	CORE
#undef	VPE
#undef	SOURCE
#undef	SOURCE_BASE
#undef	SLOTS
#undef	SPACER
#undef	BASE_OFFSET
#undef	ONE
#undef	VPE_SLOTS
#undef	GIC_BASE

END(init_gic)
