/*
 * interAptiv/start.S
 *
 *  Start of boot code for interAptiv Family of Cores
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <mips/asm.h>
#include <boot.h>
#include <mips/m32c0.h>
#include <mips/regdef.h>
#include <cps.h>
#ifdef EVA
 	#include <eva.h>
#endif

/**************************************************************************************
    R E S E T   E X C E P T I O N   H A N D L E R
**************************************************************************************/


LEAF(__reset_vector)
    la a2,     check_nmi
    mtc0	zero, C0_COUNT  	    // Clear cp0 Count (Used to measure boot time.)
    jr a2

	// Note: adding code here may conflict with board ID register at 0xbfc0010.

END(__reset_vector)


/**************************************************************************************
    B O O T   E X C E P T I O N   H A N D L E R S
**************************************************************************************/

.org 0x200 /* TLB refill, 32 bit task. */
    sdbbp
.org 0x280 /* XTLB refill, 64 bit task. */
    sdbbp
.org 0x300 /* Cache error exception. */
    sdbbp

.org 0x380 /* General exception. */
	// expecting an interprocessor interrupt
	mfc0	k1, C0_CAUSE
	ext		k0, k1, 2, 5				// extract exception code
	bne		k0, zero, other_code		// should be 0 indicating a interrupt
	ext		k0, k1, 10, 5				// extract interrupt
	li		k1, 1
	bne		k0, k1, other_code			// should be interrupt 0

	// interprocessor interrupt
    li      k0, (GIC_SH_WEDGE | GIC_BASE_ADDR_PB)
    mfc0    k1, C0_EBASE                // Get cp0 EBase
    ext     k1, k1, 0, 10				// Extract CPUNum
    addiu   k1, 0x20					// Offset to base of IPI interrupts.
    sw      k1, 0(k0)					// Clear this IPI.

    la      k0, start_test
    mfc0    k1, C0_EBASE				// Get cp0 EBase
    ext     k1, k1, 0, 10				// Extract CPUNum
    sll     k1, k1, 2					// x 4 for integer element
    addu    k0, k0, k1					// index into CMP global "C" variable start_test
    li      k1, 1
    sw      k1, 0(k0)					// Release "cpu"/vpe to execute "C" test code.
   	eret

other_code:
	// unexpected exception read the C0_CAUSE and C0_EPC so it will appear in a trace file
	// Just to make it easier to see what's in the registers
    mfc0    k1, C0_CAUSE
    mfc0	k0, C0_EPC

    sltiu	zero, zero, 0xabc1	// if using simulator stop with simulator fail condition
    // lui		a5, %hi(0xbfc00000) // Logic Analyzer triger
    // sw		zero, 0(a5)

	sdbbp 					// if probe attached break to debugger

.org 0x480 /* EJTAG debug exception (EJTAG Control Register[ProbTrap] == 0.) */
    la 		r24_boston_byte, boston_lcd_display_byte // If post boot then $24 is clobbered.
	// Write to Boston display indicating a exception
 	addu a0,  zero, r23_cpu_num
	li a1, 0x58					// X - Show  exception
	jalr r24_boston_byte

    sltiu	zero, zero, 0xabc1	// if using simulator stop with simulator fail condition
1:  b       1b  				// if no probe attached just spin

/**************************************************************************************
**************************************************************************************/
.org 0x500 /* Resume code past the boot exception vectors. */

check_nmi: // Verify we are here due to a reset (and not NMI.)
    mfc0    a0, C0_STATUS				// Read CP0 Status
    srl     a0, 19						// Shift [NMI] into LSBs.
    andi    a0, a0, 1					// Inspect CP0 Status[NMI]
    beqz    a0, verify_isa				// Branch if this is NOT an NMI exception.
    sdbbp								// Failed assertion: not NMI.

verify_isa: // Verify device ISA meets code requirements (MIPS32R2 or later.)

#if	defined(BOSTON_RAM)  // make sure ram is up before checking array
	la 	a2, init_FPGA_mem		// Initialize the ROC-it2 MC (Memory Controller.)
	jalr 	a2
#endif

    mfc0    a0, C0_CONFIG				// Read CP0 Config
    srl     a0, 10						// Shift [AT AR] into LSBs.
    andi    a3, a0, 0x18				// Inspect CP0 Config[AT]
    beqz    a3, is_mips32				// Branch if executing on MIPS32 ISA.
    sdbbp 								// Failed assertion: MIPS32R2.

is_mips32:
    andi    a3, a0, 0x07				// Inspect CP0 Config[AR]
    bnez    a3, init_common_resources	// Continue if ISA is MIPS32R2 or later.
    sdbbp								// Failed assertion MIPS32R2.
/**************************************************************************************
 What is initialized on execution depends on the core/vpe executing it.
 (A non-MT device is treated as tc0/vpe0, non-CMP device is treated as core0.)
**************************************************************************************/
init_common_resources:  // initializes resources for virtual or physical "cpu".
    la a2,init_gpr		// Fill register file with set  value.
	jalr a2

    la a2,	set_gpr_boot_values  // Fill register file boot info.
	jalr a2

    la a2,     init_cp0    // Init CP0 Status, Count, Compare, Watch*, and Cause.
    jalr a2

 	// Determine if we have a TLB
	mfc0    v1, C0_CONFIG			// read C0_Config
	ext	    v1, v1, 7, 3			// extract MT field
	li	    a3, 0x1					// load a 1 to check against
	bne	    v1, a3, done_tlb		// no tlb?

    la a2,     init_tlb    			// Generate unique EntryHi contents per entry pair.
    jalr a2

done_tlb:
    la a2,     init_gic    // Configure the global interrupt controller.
    jalr a2
    bnez    r9_vpe_num, init_done // If we are not a vpe0 then we are done.

init_core_resources:    // VPE0 on all cores will execute init_core_resources.

	// The L2 cache needs to be disabled because it has not been initialized
	// Once the Icache has been initialized below caching will be turned on for KSEG0
	// This makes initializing the Dcache and the Icache much faster since the
	// code will be cached. It would also cause the L2 cache to be used before it has
	// been initialized if it were not disabled here.
	bnez       r8_core_num, init_L1_icache	//  Only done from core 0.
    la a2,     disable_L2   // Disable L2 caches
    jalr a2

init_L1_icache:
    la a2,     init_icache // Initialize the L1 instruction cache. (Executing using I$ on return.)
    jalr a2

	#ifdef EVA
		// Only for cores built to boot in EVA mode
		// continue in uncached mode
	#else
		// The changing of Kernel mode cacheability must be done from KSEG1
		// Since the code is executing from KSEG0 It needs to do a jump to KSEG1 change
		// K0 and jump back to KSEG0
		la 		a2,		change_k0_cca
		li 		a1, 0xf
		ins		a2, a1, 29, 1 // changed to KSEG1 address by setting bit 29
		jalr	a2
	#endif

// L1 Cache icache is now enabled (if not EVA)

    la a2,     init_dcache // Initialize the L1 data cache
    jalr a2

    la a2,     init_itc    // Initialize Inter-Thread Communications unit
    jalr a2

    bnez    r8_core_num, init_sys_resources_done // Only core0/vpe0 needs to init system resources.

init_sys_resources:     // Only done by core0 vpe0.
    la a2,     init_cpc    // Initialize the CPS CPC (Cluster Power Controller.)
    jalr a2

    la a2,     init_cm     // Initialize the CPS CM (Coherence Manager.)
    jalr a2

    la a2,     copy_c2_ram // Copy "C" code and data to RAM and zero bss (uncached.)
    jalr a2

    beqz    r11_is_cps, init_vpe_next // UP Core so skip multi core stuff

	bnez       r8_core_num, release_cores	//  L2 Only done from core 0.

    la a2,     init_L2   // Initialize the unified L2 cache for non UP cores after L1 caches have been initialized
    jalr a2

    la 		a2, enable_L2   // enable L2 cache
    jalr 	a2

release_cores:

    la a2,     release_mp  // Release other cores to execute this boot code.
    jalr a2

init_sys_resources_done:		// All Cores (VPE0)

    la a2,     join_domain // Join the Coherence  domain. (OK to use D$ on return.)
    jalr a2

init_vpe_next:

    la a2,     init_vpe1   // Set up MT ASE vpe1 to execute this boot code also.
    jalr a2

init_done:

#ifdef EVA

	// For non-EXL Kernel mode - Segments CFG5 and CFG4 are directly mapped to the lower 2GB of the physical address space encompassing the 2 RAM memory blocks and the I/O space to
	// be accessed as coherent cached exclusive on write.
	// For user and supervisor modes - Segments CFG5 and CFG4 are mapped through the TLB
	li		a0, (SEGCTL_CFG4_PA_4|SEGCTL_CFG5_PA_0|SEGCTL_CFG4_AM_MUSUK|SEGCTL_CFG5_AM_MUSUK|SEGCTL_CFG4_EU_MASK|SEGCTL_CFG5_EU_MASK|SEGCTL_CFG4_C_CWBE|SEGCTL_CFG5_C_CWBE)
	mtc0 	a0, C0_SEGCTL2
	// Set CFG1 and CFG0 to be UnMapped Kernel to Physical address 0x00000000 and 0x20000000 respectively both uncached. This will be used to address the CM registers and
	// can be used for I/O devices. NOTE: This is needed to access the CM registers uncached
	li		a0, (SEGCTL_CFG2_PA_0|SEGCTL_CFG3_PA_0|SEGCTL_CFG2_AM_UK|SEGCTL_CFG3_AM_UK|SEGCTL_CFG2_EU_MASK|SEGCTL_CFG3_EU_MASK|SEGCTL_CFG2_C_UC|SEGCTL_CFG3_C_WB)
    mtc0 	a0, C0_SEGCTL1
	ehb

#endif

    // Prepare for eret to main (sp and gp set up per vpe in init_gpr).
    la      ra, all_done	// If main return then go to all_done:.
    la      a1, main
    mtc0    a1, C0_ERRPC 			// ErrorEPC

    // initialize global variable num_cores.
    la      a1, num_cores			// load address of num_cores into a1
    #ifndef EVA
    	li		a2, 0xFFFFFFFF
    	ins     a1, a2, 29, 1  // convert it to a Uncached kseg1 address
    #endif
    add     a0, r19_more_cores, 1	// set a0 to the total number of cores
    sw      a0, 0(a1)				// store number of cores to address of num_cores

    // Prepare arguments for main()
    move    a0, r23_cpu_num			// main(arg0) is the "cpu" number (cp0 EBase[CPUNUM].)
    move    a1, r8_core_num			// main(arg1) is the core number.
    move    a2, r9_vpe_num			// main(arg2) is the vpe number.
    addiu   a3, r20_more_vpes, 1	// main(arg3) is the number of vpe on this core.

    eret    // Exit reset exception handler for this vpe and start execution of main().

/**************************************************************************************
**************************************************************************************/
all_done:
	sltiu $0, $0, 0xabc1	# simulator fail condition
	nop                     # this nop prevents assembler from putting sltiu into delay slot below
	// Looks like main returned. Just busy wait spin.
    b       all_done
